#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: CMFNeoBoard.py,v 1.12 2003/03/30 13:08:03 wankyu Exp $
Description

    CMFNeoBoard main
"""

from Globals import InitializeClass, DTMLFile
from Acquisition import aq_base

from Products.CMFCore import CMFCorePermissions, utils
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.Expression import Expression as BaseExpression
from Products.CMFCore.ActionInformation import ActionInformation as BaseAction
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware

from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from Products.CMFCore.utils import getToolByName, _getViewFor
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.NeoBoard.NeoBoard import NeoBoard

import os

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

factory_type_information = ({
                              'id'             : 'CMFNeoBoard'
                            , 'meta_type'      : 'CMFNeoBoard'
                            , 'description'    : 'CMFNeoBoard is a threaded message board product.'
                            , 'icon'           : 'neoboard.gif'
                            , 'product'        : 'NeoBoard'
                            , 'factory'        : 'addCMFNeoBoard'
                            , 'filter_content_types' : 1
                            , 'allowed_content_types':['NeoBoard Article', 'NeoBoard Notice Article']
                            , 'immediate_view' : ''
                             , 'actions'        :
                                ( 
                                  { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : ''
                                  , 'permissions'   :
                                   ( NeoPortalView,)
                                  , 'category'      : 'neoboard'
                                  }
                                  
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'neoboard_edit_form'                                  
                                  
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  , 'visible'       : 0
                                  }                                  
                                  
                                  , { 'id'            : 'metadata'
                                  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   :
                                   ( NeoPortalAddPortalContent,)
                                  , 'category'      : 'neoboard'
                                  },
                                                                    
                                  
                              )
                            },
                            )

def addCMFNeoBoard(self 
            , id
            , title=''
            , description=''
            , REQUEST=None):
    
    """ 
        Add a CMFNeoBoard
    """

    instance = CMFNeoBoard( id=id )

    instance.enabled = 1
    instance.acl_user_auth = 1
    instance.auto_approve_article = 1
        
    self._setObject(id, instance)
        
    if REQUEST is not None:
        return self.manage_main( self, REQUEST=REQUEST )

class CMFNeoBoard( CMFCatalogAware
                 , NeoBoard
                 , PortalContent
                 , PortalFolder
                 , DefaultDublinCoreImpl
                 , ActionProviderBase ):
    """
        CMFNeoBoard
    """

    meta_type = 'CMFNeoBoard'
    
    meta_types=(
        {'name': 'NeoBoard Article', 'action':'neoboard_article_post_form', 'permission': 'Add NeoBoard Article'},
        {'name': 'NeoBoard Notice Article', 'action':'neoboard_article_post_form', 'permission': 'Add NeoBoard Article'},
    )
    
    portal_type = 'CMFNeoBoard'
    
    all_meta_types=meta_types    
    
    _actions = [
                BaseAction( id='expanded_view'
                              , title='+View'
                              , description='Normal View'
                              , action="python: object.getNeoBoard().absolute_url() + '?expand_all=1'"
                              , permissions=( NeoPortalView, )
                              , category='neoboard'
                              , condition='python: object is not None'
                              , visible=1), 

                BaseAction( id='collapsed_view'
                              , title='-View'
                              , description='Normal View'
                              , action="python: object.getNeoBoard().absolute_url() + '?expand_all=0'"
                              , permissions=( NeoPortalView, )
                              , category='neoboard'
                              , condition='python: object is not None'
                              , visible=1), 
                
                BaseAction( id='edit_neoboard'    
                              , title='Config'
                              , description='Edit this object instance.'
                              , action=BaseExpression(
                                 text='string:${object/absolute_url}/?neoboard_manager_form=neoboard_edit_form')
                              , permissions=( NeoPortalAddPortalContent ,)
                              , category='neoboard'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1), 

                 BaseAction( id='edit_categories'
                              , title='Categories'
                              , description='Edit object Categories.'
                              , action=BaseExpression(
                                 text="python: object.getNeoBoard().absolute_url() + '?neoboard_manager_form=neoboard_categories_edit_form'")
                              , permissions=( NeoPortalAddPortalContent ,)
                              , category='neoboard'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1),                              


                 BaseAction( id='edit_managers'
                              , title='Managers'
                              , description='Edit object Managers.'
                              , action=BaseExpression(
                                 text="python: object.getNeoBoard().absolute_url() + '?neoboard_manager_form=neoboard_managers_edit_form'")
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='neoboard'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1),                              

                 BaseAction( id='batch_delete'
                              , title='Delete'
                              , description='Delete selected articles.'
                              , action='string:${object/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='batch'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1),                               

                 BaseAction( id='batch_approve'
                              , title='Approve'
                              , description='Approve selected articles.'
                              , action='string:${object/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='batch'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1), 

                 BaseAction( id='batch_disapprove'
                              , title='Disapprove'
                              , description='Disapprove selected articles.'
                              , action='string:${object/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='batch'
                              , condition='python: object is not None and object.isNeoBoardManager()'
                              , visible=1), 
                              
                 BaseAction( id='batch_move'
                              , title='Move'
                              , description='Move selected articles.'
                              , action='string:${object/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='batch'
                              , condition="python: object is not None and (object.isNeoBoardManager() and len(object.getEnabledNeoBoards()) >= 1)"
                              , visible=1), 

                 BaseAction( id='batch_copy'
                              , title='Copy'
                              , description='Copy selected articles.'
                              , action='string:${object/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='batch'
                              , condition="python: object is not None and (object.isNeoBoardManager() and len(object.getEnabledNeoBoards()) >= 1)"
                              , visible=1),                               
                              
                 BaseAction( id='post'
                              , title='Post'
                              , description='Post an article.'
                              , action="python: object.getNeoBoard().absolute_url() + '/?neoboard_form=neoboard_article_post_form'"
                              , permissions=( NeoPortalView ,)
                              , category='neoboard'
                              , condition="python: not object.isReadOnly() or object.isNeoBoardManager()"
                              , visible=1), 

                 BaseAction( id='edit'
                              , title='Edit'
                              , description='Edit this article.'
                              , action="string:neoboard_article_edit_form"
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='article'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticle', 0) and object.isNeoBoardManager()"
                              , visible=1),                               

                 BaseAction( id='reply'
                              , title='Reply'
                              , description='Reply to this article.'
                              , action="string:neoboard_article_reply_form"
                              , permissions=( NeoPortalView ,)
                              , category='article'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticle', 0) and ( object.isNeoBoardManager() or object.isApproved())"
                              , visible=1), 

                 BaseAction( id='modify'
                              , title='Modify'
                              , description='Mofidy this article.'
                              , action="string:neoboard_article_modify_form"
                              , permissions=( NeoPortalView,)
                              , category='article'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticle', 0) and ( object.isNeoBoardManager() or object.isApproved() )"
                              , visible=1), 
                              
                 BaseAction( id='delete'
                              , title='Delete'
                              , description='Delete this article.'
                              , action="string:neoboard_article_delete_form"
                              , permissions=( NeoPortalView,)
                              , category='article'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticle', 0) and ( object.isNeoBoardManager() or object.isApproved() )"
                              , visible=1), 

                 BaseAction( id='log'
                              , title='Log'
                              , description='View daily access logs for this article.'
                              , action='string:neoboard_article_access_log'
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='article'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticle', 0) and ( object.isNeoBoardManager() or object.isApproved() ) and object.getNeoPortalNumAccessLogEntries() > 0"
                              , visible=1), 
                              

                 BaseAction( id='modify'
                              , title='Modify'
                              , description='Mofidy this comment.'
                              , action="string:neoboard_comment_modify_form"
                              , permissions=( NeoPortalView, )
                              , category='comment'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticleComment' ,0 )"
                              , visible=1), 
                              
                 BaseAction( id='delete'
                              , title='Delete'
                              , description='Delete this comment.'
                              , action="string:neoboard_comment_delete_form"
                              , permissions=( NeoPortalView, )
                              , category='comment'
                              , condition="python: object is not None and getattr(object,'isNeoBoardArticleComment', 0)"
                              , visible=1), 
                              
                 BaseAction( id='metadata'
                              , title='Metadata'
                              , description='Metadata Edit'
                              , action="string:metadata_edit_form"
                              , permissions=( NeoPortalAddPortalContent, )
                              , category='neoboard'
                              , condition="python: object is not None and object.isNeoBoardManager()"
                              , visible=1),                                                
                ]     
    
    manage_options = ( DefaultDublinCoreImpl.manage_options 
                     + ActionProviderBase.manage_options
                     + NeoBoard.manage_options )                         
    
    security = ClassSecurityInfo()
    
    def __init__( self, id ):
        DefaultDublinCoreImpl.__init__(self)        
        self.setFormat(self.getDefaultNeoBoardFormat())
        NeoBoard.__init__(self, id=id)
        if self.getMailHosts():
            self.mailhost = self.getMailHosts()[0]
    
    def __call__( self ):
        """
            Return default view
        """
        
        if self.isNeoPortalObjectUpgradeNeeded(): return self.getNeoPortalContentUpgradeForm()
        
        view = self.neoboard_main
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    index_html = __call__

    security.declareProtected( NeoPortalManageProperties, 'neoboard_edit_form' )
    def neoboard_edit_form( self, REQUEST=None ):
        """
            Return edit form
        """ 

        if REQUEST is not None:            
            REQUEST.set('neoboard_manager_form', 'neoboard_edit_form')
            
        return self.neoboard_main( REQUEST=REQUEST )

    security.declarePublic( 'Format' )
    def Format( self ):
        """
            Return Renderable Format
        """ 
        return self.format

    security.declareProtected( NeoPortalManageProperties, 'setFormat' )
    def setFormat( self, format ):
        """
            Set Format
        """ 
        self.format = format    

    security.declarePublic( 'getNeoBoardCharset' )
    def getNeoBoardCharset( self ):
        """
            Return the charset of the NeoBoard instance: CMF Plone's default_charset is returned when found
        """ 
        
        try:
            return self.portal_properties.site_properties.default_charset
        except:
            return NeoBoard.getNeoBoardCharset( self )

    security.declarePublic( 'getAuthenticatedNeoPortalUser' )
    def getAuthenticatedNeoPortalUser(self):
        """
            Wrapper: CMFNeoBoard uses the membership tool
        """ 


        mtool = getToolByName( self, 'portal_membership' )    
        return mtool.getAuthenticatedMember()


    security.declarePublic('getMailHost')
    def getMailHost(self):
        """ 
            Return the CMF's mailhost instance
        """
        try:
            return getattr(self, 'MailHost')
        except AttributeError:
            return None
            
    security.declarePrivate('_get_id')                
    def _get_id(self, id):
        """
            New ID's for moved articles
        """     
    
        return NeoBoard._get_id(self, id)

    security.declarePrivate( 'listActions' )
    def listActions(self, info=None):
        """ 
            Return list of actions duplicating CMF compatible actions
        """
        #
        # FIXME: there'd be a better way...
        #

        new_actions = [ action  for action in self._cloneActions() if action.category=='neoboard' ]
        
        for action in new_actions: action.category = 'object'
        
        return ( tuple( self._actions ) + tuple( new_actions ) )

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """
            Tell if self is being copied
        """             

        if self.useFileSystem():
            # When adding a new NeoBoard, use_filesystem is not set to true, not matter what,
            # at the time of its instantiation.
            # If it's set, this instance is being copied or moved!
        
            if self.archive_path.endswith( os.sep ):
                self.archive_path = self.archive_path[:-1]
                
            self.archive_path =  self.archive_path + '_TEMP_ARCHIVE_' + self.getId() + os.sep
            self._temp_archive_set = 1
    
            os.mkdir( self.archive_path, 0755 )
                            
        return CMFNeoBoard.inheritedAttribute ("manage_afterAdd")(self, item, container)  

InitializeClass( CMFNeoBoard )
