#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY log                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: Install.py,v 1.9 2003/04/02 08:50:40 wankyu Exp $
Description

   CMFNeoBoard Installer
"""
from Globals import package_home
from Acquisition import aq_get, aq_parent, aq_base
from Products.NeoBoard import CMFNeoBoard, neoboard_globals
from Products.NeoBoard.CMFNeoBoard import CMFNeoBoard, factory_type_information
from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName, minimalpath
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import createDirectoryView


def install(self):
    """
        Sqeeuze in CMFNeoBoard Skins/Types
    """

    log = StringIO()
    types_tool = getToolByName(self, 'portal_types')
    skins_tool = getToolByName(self, 'portal_skins')

    for fti in factory_type_information:
        if not hasattr( aq_base( types_tool ), fti['id']):
            cfm = apply(ContentFactoryMetadata, (), fti)
            types_tool._setObject(fti['id'], cfm)
                           
            log.write('Registered CMFNeoBoard with the types tool...\n')
        else:
            types_tool._delObject(fti['id'])        
            cfm = apply(ContentFactoryMetadata, (), fti)
            types_tool._setObject(fti['id'], cfm)        
        
            log.write('Type "%s" already exists in the types tool...overwritten.\n' % (
                fti['id']))


    if 'neoboard_skins' not in skins_tool.objectIds():
        fp = minimalpath("%s/neoboard_skins" % package_home(neoboard_globals))
        createDirectoryView(skins_tool, fp, 'neoboard_skins')        
                
        log.write("Added 'neoboard_skins' directory views to portal_skins\n")

    skins = skins_tool.getSkinSelections()
    
    for skin in skins:
        path=[elem.strip() for elem in skins_tool.getSkinPath(skin).split(',')]        
        if 'neoboard_skins/default/icons' not in path:
            for n_skin in ['neoboard_skins/default/icons'
                     , 'neoboard_skins/default/images'
                     , 'neoboard_skins/default/templates'
                     , 'neoboard_skins/default/scripts'                     
                     , 'neoboard_skins/default/forms']:
                try:
                    path.insert( path.index('custom') + 1, n_skin)
                except:
                    path.append(n_skin)

            path = ', '.join( path )                
            skins_tool.addSkinSelection(skin, path)
            log.write("Added NeoBoard Skins to %s skins\n" % skin)
                            
        else:
            log.write('Skin %s already has NeoBoard Skins\n' % skin)
            
    actions_tool = getToolByName( self, 'portal_actions')

    nb_actions = [ 'nb_expanded_view', 'nb_collapsed_view', 'nb_edit_neoboard', 'nb_edit_categories', 'nb_edit_managers', 'nb_article_post' ]
    
    actions = actions_tool._cloneActions()
    new_actions = []
    
    for action in actions:
        if action.id in nb_actions:
            continue
        new_actions.append(action)

    actions_tool._actions = tuple( new_actions )

    actions_tool.addAction('nb_expanded_view', '+View', "python: object.getNeoBoard().absolute_url() + '?expand_all=1'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board')", 'View', 'object')
    actions_tool.addAction('nb_collapsed_view', '-View', "python: object.getNeoBoard().absolute_url() + '?expand_all=0'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board')", 'View', 'object')
    actions_tool.addAction('nb_edit_neoboard', 'Config', "python: object.getNeoBoard().absolute_url() + '?neoboard_manager_form=neoboard_edit_form'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board') and object.isNeoBoardManager()", 'Modify portal content', 'object')
    actions_tool.addAction('nb_edit_categories', 'Categories', "python: object.getNeoBoard().absolute_url() + '?neoboard_manager_form=neoboard_categories_edit_form'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board') and object.isNeoBoardManager()", 'Modify portal content', 'object')
    actions_tool.addAction('nb_edit_managers', 'Managers', "python: object.getNeoBoard().absolute_url() + '?neoboard_manager_form=neoboard_categories_edit_form'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board') and object.isNeoBoardManager()", 'Modify portal content', 'object')
    actions_tool.addAction('nb_article_post', 'Post', "python: object.getNeoBoard().absolute_url() + '?neoboard_form=neoboard_article_post_form'", "python: hasattr( object, 'neoboard_article') and not hasattr(object, 'neoportal_discussion_board')", 'View', 'object') 
            
    log.write('Added NeoBoard article actions to the actions tool\n')            
    
    nb_types = [ nb for id, nb in self.ZopeFind( self, search_sub=1 ) if ( nb.meta_type == 'NeoBoard' or nb.meta_type == 'CMFNeoBoard' ) and nb.isNeoPortalObjectUpgradeNeeded() ]
                    
    if not nb_types:
        log.write('*****************************************************\n\n')
        log.write("Attempted to upgrade NeoBoard instances but no upgradable instance found. Exiting...\n")
    else:
        log.write('*****************************************************\n\n')
        log.write("%d NeoBoard Instance(s) found.\n\n\n" % len( nb_types ))
        log.write('*****************************************************\n\n')
        
        upgraded_instances = 0
        for t in nb_types:
            t.selfUpgradeNeoPortalContent()
            upgraded_instances = upgraded_instances + 1

        from Products.NeoBoard import NeoBoard
        log.write("%d Instance(s) upgraded to %s.\n\n\n" % ( upgraded_instances, NeoBoard.__NEOPORTAL_CLASS_VERSION__) )
        log.write('*****************************************************\n\n')
            
    return log.getvalue()



    
