#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY log                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: Patch.py,v 1.5 2002/12/13 01:44:26 wankyu Exp $
Description

   NeoBoard Article patch for 1.03 instances:
   
    You should run this external method if you want to upgrade NeoBoard 1.03 instances to 1.1b.
"""
from Globals import PersistentMapping
from cStringIO import StringIO

def run(self):
    """
       Seek and destroy old attachments!
    """

    log = StringIO()
    
    # NeoBoard 1.03 has an attribute '_upgraded_up_to' set to '1.0b'

    neoboards = [ nb for id, nb in self.ZopeFind( self, obj_metatypes = ['NeoBoard', ], search_sub=1 ) if hasattr(nb, '_upgraded_up_to') and nb._upgraded_up_to == '1.0b' ]
    
    if not neoboards:
        log.write("No NeoBoard 1.03 instance has been found. Exiting...\n")
    else:
        log.write("%d NeoBoard 1.03 instance(s) found.\n\n\n" % len( neoboards ))
        log.write('*****************************************************\n\n')
        
        for nb in neoboards:
            nb._nb_patched = 1
            log.write("Patching %s articles for 1.1b upgrade.....\n" % nb.getId() )
        
            articles = [ article for id, article in self.ZopeFind( nb, obj_metatypes = ['NeoBoard Article', 'NeoBoard Notice Article'], search_sub=1 ) ]    
            log.write("%d articles found.\n" % len( articles ))
            
            processed = 0
            for article in articles:
                if not article.hasAttachment(): continue    # only articles with old attachment instances are of interest here
                
                article._nb_attachments = []
                attachments = article.getAttachments()
                for attachment in attachments:
                    info = {}
                    if hasattr( attachment, 'title' ):
                        info['title'] = attachment.title
                    if hasattr( attachment, 'description' ):
                        info['description'] = attachment.description
                    if hasattr( attachment, 'file_path' ):
                        info['filepath'] = attachment.file_path
                    if hasattr( attachment, 'content_type' ):
                        info['content_type'] = attachment.content_type
                    
                    article._nb_attachments.append( info )
                    article._p_changed = 1
                    
                processed = processed + 1

            log.write("%d article(s) patched for 1.1b upgrade. Only articles with attachments need to be patched.\n\n" % processed )

        log.write('\n\n\n')
        log.write('*****************************************************\n')
        log.write( 'Patch complete. Now delete the old NeoBoard package from the Products directory and install 1.1b package.\n') 
        log.write( 'Visit each patched NeoBoard instance by its absolute_url + index_html public interface\n and follow the instructions NeoBoard 1.1b presents.\n') 
                            
    return log.getvalue()



    
