#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoard.py,v 1.39 2003/04/02 07:58:36 wankyu Exp $
Description

    NeoBoard Main
"""

from Globals import InitializeClass, PersistentMapping, DTMLFile
from AccessControl import ClassSecurityInfo, getSecurityManager, Unauthorized
from DateTime.DateTime import DateTime
from Acquisition import ImplicitAcquisitionWrapper, aq_parent, aq_base, aq_inner
from OFS.SimpleItem import SimpleItem

__PREVIOUS_NEOPORTAL_CLASS_VERSION__ = '1.1b2'
__NEOPORTAL_CLASS_VERSION__ = '1.1' 

#
# For skin magics;-)
#

from OFS.ObjectManager import ObjectManager
try:
    superGetAttr = ObjectManager.__getattr__
except:
    try:
        superGetAttr = ObjectManager.inheritedAttribute( '__getattr__' )
    except:
        superGetAttr =None

from Products.NeoPortalLibrary.NP_Utils import isEmailValid, NeoPortalBaseExpression, NeoPortalBaseAction
from Products.NeoPortalLibrary.NP_Mailer import NeoPortalMailer
from Products.NeoPortalLibrary.NP_BaseExternalFile import checkFilePath, getAllowedFileBases

from Products.NeoBoard.NeoBoardBase import NeoBoardBase
from Products.NeoBoard.NeoBoardArticle import NeoBoardArticle, NeoBoardNoticeArticle
from Products.NeoBoard.NeoBoardArticleAttachment import NeoBoardArticleAttachment

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import os

def getNeoBoardInitSkinFolder( self):
    """
        Return the first element from the list of available NeoBoard Skin Folders when adding NeoBoard 
    """ 
    try:
        skin_folders = self.objectValues( 'NeoBoard Skin Folder', )
        if skin_folders and skin_folders[0].meta_type == 'NeoBoard Skin Folder':
            return skin_folders[0]
        else:
            return None
    except:
        return None

stylesheet = DTMLFile( 'neoboard_skins/default/templates/neoboard_init_stylesheet', globals() )
manage_addNeoBoardForm =  DTMLFile( 'dtml/NeoBoardAdd', globals() )

_archive_marker = '.__neoboard_archive_path__'

def _mark_archive_path(path, board_url):
    """
        Mark archive path as 'in use by this board' to prevent 
        other boards from accidentally choosing the same archive path
    """
    if not path.endswith( os.sep ):
        path = path + os.sep
    
    fname = path + _archive_marker
    if os.path.isfile( fname ):
        f = open( fname, 'r' )
        used_by = f.read().strip()
        if used_by == board_url:
            return 1
        else:
            return 'Arhicive Path Used by another NeoBoard: %s' % used_by
    else:        
        f = open( fname,'w' )
        f.write( board_url )
        f.close()
        return 1

from Products.PageTemplates.Expressions import getEngine
from Products.PageTemplates.Expressions import SecureModuleImporter

def createExprContext( folder, neoboard, object, auth_user ):
    """
        For CMF Compatibility
    """
    if object is None:
        object_url = ''
    else:
        object_url = object.absolute_url()
    if auth_user is None:
        user = None
    else:
        user = auth_user

    if getattr( object, 'isNeoBoardArticle', 0 ) and object.isNeoBoardArticle():
        article = object
        article_url = object_url
    else:
        article = None
        article_url = ''

    if getattr( object, 'isNeoBoardArticleComment', 0 ) and object.isNeoBoardArticleComment():
        comment = object
        comment_url = object_url
    else:
        comment = None
        comment_url = ''

    if getattr( object, 'isNeoBoardArticleAttachment', 0 ) and object.isNeoBoardArticleAttachment():
        attachment = object
        attachment_url = object_url
    else:
        attachment = None
        attachment_url = ''
    
    
    REQUEST = getattr( object, 'REQUEST', None )
    QUERY_STRING = getattr(REQUEST, 'QUERY_STRING', '' )
    data = {
        'object_url':   object_url,
        'article_url':  article_url,        
        'comment_url':  comment_url, 
        'attachment_url':  attachment_url, 
        'folder_url':   folder.absolute_url(),
        'neoboard_url': neoboard.absolute_url(),
        'object':       object,
        'article':  article,        
        'comment':  comment, 
        'attachment':  attachment,         
        'folder':       folder,
        'neoboard':     neoboard,
        'nothing':      None,
        'request':      REQUEST,
        'query_string': QUERY_STRING,
        'modules':      SecureModuleImporter,
        'user':         user,
        }
        
    return getEngine().getContext( data ) 
        
def manage_addNeoBoard( self 
            , id
            , title=''
            , description=''
            , REQUEST=None ):
    
    """ 
        Add a NeoBoard instance
    """

    instance = NeoBoard( id=id
                       , title=title
                       , description=description )
    
    self._setObject( id, instance )
        
    if REQUEST is not None:
        return self.manage_main( self, REQUEST=REQUEST )

class NeoBoard( NeoBoardBase ):
    """
        NeoBoard
    """

    neoboard = 1

    __NEOPORTAL_CLASS_VERSION__  = __NEOPORTAL_CLASS_VERSION__

    meta_type='NeoBoard'
    icon='misc_/NeoBoard/neoboard_icon'     

    meta_types=(
        {'name': 'NeoBoard Article', 'action':'neoboard_article_post_form', 'permission': 'Add NeoBoard Article'},
        {'name': 'NeoBoard Notice Article', 'action':'neoboard_article_post_form', 'permission': 'Add NeoBoard Article'},
    )
    
    all_meta_types=meta_types
    
    _actions = [

                NeoPortalBaseAction( id='expanded_view'
                              , title='+View'
                              , description='Normal View'
                              , action='string:${neoboard/absolute_url}?expand_all=1'
                              , permissions=( NeoPortalView, )
                              , category='neoboard'
                              , condition=''
                              , visible=1), 

                NeoPortalBaseAction( id='collapsed_view'
                              , title='-View'
                              , description='Normal View'
                              , action='string:${neoboard/absolute_url}?expand_all=0'
                              , permissions=( NeoPortalView, )
                              , category='neoboard'
                              , condition=''
                              , visible=1), 

     
                NeoPortalBaseAction( id='edit_neoboard'    
                              , title='Edit NeoBoard'
                              , description='Edit this NeoBoard instance.'
                              , action='string:${neoboard/absolute_url}?neoboard_manager_form=neoboard_edit_form'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='neoboard'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1), 

                 NeoPortalBaseAction( id='edit_categories'
                              , title='Edit Categories'
                              , description='Edit NeoBoard Categories.'
                              , action='string:${neoboard/absolute_url}?neoboard_manager_form=neoboard_categories_edit_form'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='neoboard'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1),                              


                 NeoPortalBaseAction( id='edit_managers'
                              , title='Edit Managers'
                              , description='Edit NeoBoard Managers.'
                              , action='string:${neoboard/absolute_url}/?neoboard_manager_form=neoboard_managers_edit_form'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='neoboard'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1),                              

                 NeoPortalBaseAction( id='batch_delete'
                              , title='Delete'
                              , description='Delete selected articles.'
                              , action='string:${neoboard/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='batch'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1),                               

                 NeoPortalBaseAction( id='batch_approve'
                              , title='Approve'
                              , description='Approve selected articles.'
                              , action='string:${neoboard/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='batch'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1), 

                 NeoPortalBaseAction( id='batch_disapprove'
                              , title='Disapprove'
                              , description='Disapprove selected articles.'
                              , action='string:${neoboard/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='batch'
                              , condition='python: neoboard.isNeoBoardManager()'
                              , visible=1), 
                              
                 NeoPortalBaseAction( id='batch_move'
                              , title='Move'
                              , description='Move selected articles.'
                              , action='string:${neoboard/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='batch'
                              , condition="python: neoboard.isNeoBoardManager() and len(neoboard.getEnabledNeoBoards() ) >= 1"
                              , visible=1), 

                 NeoPortalBaseAction( id='batch_copy'
                              , title='Copy'
                              , description='Copy selected articles.'
                              , action='string:${neoboard/absolute_url}/manage_batchAction'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='batch'
                              , condition="python: neoboard.isNeoBoardManager() and len(neoboard.getEnabledNeoBoards() ) >= 1"
                              , visible=1),                               
                              
                 NeoPortalBaseAction( id='post'
                              , title='Post'
                              , description='Post an article.'
                              , action="string:${neoboard/absolute_url}?neoboard_form=neoboard_article_post_form"
                              , permissions=( NeoPortalView, )
                              , category='neoboard'
                              , condition="python: not neoboard.isReadOnly() or neoboard.isNeoBoardManager()"
                              , visible=1), 

                 NeoPortalBaseAction( id='edit'
                              , title='Edit'
                              , description='Edit this article.'
                              , action="string:neoboard_article_edit_form"
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='article'
                              , condition="python: neoboard.isNeoBoardManager() and article is not None"
                              , visible=1),                               

                 NeoPortalBaseAction( id='reply'
                              , title='Reply'
                              , description='Reply to this article.'
                              , action="string:neoboard_article_reply_form"
                              , permissions=( NeoPortalView, )
                              , category='article'
                              , condition="python: article is not None and ( neoboard.isNeoBoardManager() or article.isApproved() )"
                              , visible=1), 

                 NeoPortalBaseAction( id='modify'
                              , title='Modify'
                              , description='Mofidy this article.'
                              , action="string:neoboard_article_modify_form"
                              , permissions=( NeoPortalView, )
                              , category='article'
                              , condition="python: article is not None and ( neoboard.isNeoBoardManager() or article.isApproved() )"
                              , visible=1), 
                              
                 NeoPortalBaseAction( id='delete'
                              , title='Delete'
                              , description='Delete this article.'
                              , action="string:neoboard_article_delete_form"
                              , permissions=( NeoPortalView, )
                              , category='article'
                              , condition="python: article is not None and ( neoboard.isNeoBoardManager() or article.isApproved() )"
                              , visible=1), 

                 NeoPortalBaseAction( id='log'
                              , title='Log'
                              , description='View daily access logs for this article.'
                              , action='string:neoboard_article_access_log'
                              , permissions=( NeoPortalAddPortalContent,)
                              , category='article'
                              , condition="python: article is not None and article.isApproved() and article.getNeoPortalNumAccessLogEntries() > 0"
                              , visible=1), 

                 NeoPortalBaseAction( id='modify'
                              , title='Modify'
                              , description='Mofidy this comment.'
                              , action="string:neoboard_comment_modify_form"
                              , permissions=( NeoPortalView, )
                              , category='comment'
                              , condition="python: comment is not None"
                              , visible=1), 
                              
                 NeoPortalBaseAction( id='delete'
                              , title='Delete'
                              , description='Delete this comment.'
                              , action="string:neoboard_comment_delete_form"
                              , permissions=( NeoPortalView, )
                              , category='comment'
                              , condition="python: comment is not None"
                              , visible=1), 
                              
                ] 
    
    _max_num_attachments = 10
    _max_thread_depth_hspace = 20   # pixel

    _v_neoboard_skindata = None

    security = ClassSecurityInfo()
    
    def __init__( self, id, title='', description='' ):
        
        NeoBoardBase.__init__( self, id=id)
            
        self._edit()        
        self.setNeoBoardSkin( 'default' )
        self.setFormat( self.getDefaultNeoBoardFormat() )
        self.managers = PersistentMapping()
        self.posting_categories = PersistentMapping()    
        self.effective_date = DateTime()
        
        self._setCurrentClassVersion( __NEOPORTAL_CLASS_VERSION__ )
        self._edit( title=title
                  , description=description
                  , access_log_access_trace_key=['Date',]
                  , content_rating_access_trace_key=['IP Address','Username'] )
        
    def __call__( self, REQUEST=None ):
        """
            Default View
        """
        
        if self.isNeoPortalObjectUpgradeNeeded(): return self.getNeoPortalContentUpgradeForm()
        
        if REQUEST is None:
            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
            else:
                return ''
              
        view = self.neoboard_main
        if getattr( aq_base( view ), 'isDocTemp', 0 ):
            return apply( view, ( self, self._cookREQUEST( REQUEST ) ) )
        else:
            return view()

    index_html = __call__                
      
    security.declarePublic( 'isNeoBoardCollapsable' )
    def isNeoBoardCollapsable( self, REQUEST=None ):
        """
            Validate permissions
        """ 

        try:
            if REQUEST is None:
                    REQUEST = self.REQUEST
            
            if not REQUEST.has_key('expand_all') or ( REQUEST.has_key('expand_all') and int( REQUEST['expand_all'] ) ):
                return 1
        except:        
            pass

        return 0

    security.declarePublic( 'checkNeoBoardPermission' )
    def checkNeoBoardFormPermission( self, manager_form=0 ):
        """
            Validate permissions
        """ 

        if self.useACLUserAuth() and self.isAnonymousNeoPortalUser() and not self.anonymousPostingAllowed():
            raise Unauthorized
        
        if manager_form and not self.isNeoBoardManager():
            raise Unauthorized

        return 1

    security.declarePublic( 'getEmailFromName' )
    def getEmailFromName( self):
        """
            Return from email name
        """ 
        if not hasattr( self, 'email_from_name' ):
            self.email_from_name = ''
        
        return self.email_from_name

    security.declarePublic( 'getEmailFromAddress' )
    def getEmailFromAddress( self):
        """
            Return from email address
        """ 
        if not hasattr( self, 'email_from_address' ):
            self.email_from_address = ''
                    
        return self.email_from_address
        
    security.declarePublic( 'isNeoBoard' )
    def isNeoBoard( self):
        """
            Tell if self is NeoBoard instance
        """ 
            
        return self.meta_type == NeoBoard.meta_type
        
    security.declarePublic( 'listFilteredActionsFor' )
    def listFilteredActionsFor( self, object=None ):
        """
            Return all actions available to user: for CMF compatibility
        """
        neoboard = aq_parent( aq_inner( self ) )

        if object is None or not hasattr( object, 'aq_base' ):
            folder = neoboard
        else:
            folder = object
            while folder is not None:
                if getattr(aq_base(folder), 'isPrincipiaFolderish', 0):
                    break
                else:
                    folder = aq_parent(aq_inner(folder) )
                    
        ec = createExprContext( folder, self.getNeoBoard(), object, auth_user=self.getAuthenticatedNeoPortalUser() )

        filtered_actions={ 'neoboard':[],
                          'article':[],
                          'comment':[],
                          'batch':[],
                          }

        for action in [ ai.getAction( ec ) for ai in self._actions if ai.testCondition( ec ) ]:
            category = action['category']
            permissions = action.get( 'permissions', None )
            visible = action.get( 'visible', 1 )
            if not visible:
                continue
            verified = 0
            if not permissions:
                verified = 1
            else:
                if ( object is not None and
                    ( category.startswith( 'article' ) or category.startswith( 'attachment' ) or 
                     category.startswith( 'comment' ) ) ):
                    context = object
                else:
                    context = folder # NeoBoard

                for permission in permissions:
                    checkPermission=getSecurityManager().checkPermission
                    if checkPermission( permission, context ):
                        verified = 1
                        break
            if verified:
                catlist = filtered_actions.get( category, None )
                if catlist is None:
                    filtered_actions[category] = catlist = []
                if not action in catlist:
                    catlist.append(action)

        return filtered_actions

    security.declarePublic( 'listFilteredActions' )
    listFilteredActions = listFilteredActionsFor  
    
    security.declarePublic( 'getNeoBoardDate' )
    def getNeoBoardDate( self, dt_obj):
        """
            Return NeoBoard Date String from dt_obj
        """ 
        try:
            d = DateTime( str( dt_obj ) )
        except:
            # not a DateTime object
            return dt_obj
            
        return d.strftime( '%Y-%m-%d' )

    security.declarePublic( 'getNeoBoardDateTime' )
    def getNeoBoardDateTime( self, dt_obj):
        """
            Return NeoBoard DateTime String from dt_obj
        """ 
        try:
            d = DateTime( str( dt_obj ) )
        except:
            # not a DateTime object
            return dt_obj
        
        return d.strftime( '%Y-%m-%d %H:%M:%S' )    
   
    security.declarePublic( 'getNeoBoard' )                
    def getNeoBoard( self):
        """
            Return Self
        """     
        return self.getNeoPortalElementContainer()

    security.declarePublic( 'getNeoBoardURL' )
    def getNeoBoardURL( self):
        """
            Return URL
        """     
        return self.getNeoBoard().absolute_url()

    security.declarePublic( 'getEnabledNeoBoards' )                
    def getEnabledNeoBoards( self):
        """
            Return a list of enabled NeoBoards in the same folder
        """     
        return [ neoboard for neoboard in self.aq_inner.aq_parent.objectValues( self.meta_type ) if neoboard.isNeoBoardEnabled() and neoboard.getId() != self.getId() ]

    
    security.declarePublic( 'getNeoBoards' )                
    def getNeoBoards( self):
        """
            Return a list of NeoBoards in the same folder
        """     
        return [ neoboard for neoboard in self.aq_inner.aq_parent.objectValues( self.meta_type ) if neoboard.getId() != self.getId() ]


    security.declarePublic( 'search' )
    def search(self, indexes=None, keywords=None ):
        """
            NeoBoard search interface:
            
                - indexes: indexes to search on
                
                - keywords: list of keywords
        """
        
        if indexes is None or keywords is None:
            return []

        results = []
        for skey in keywords:
            for idx in indexes:
                print '%s:%s' % (idx, skey)
                results.extend( self.searchResults( { idx:skey }
                                , meta_type=NeoBoardArticle.meta_type
                                , isTempNeoBoardArticle=0
                                , sort_on='getSortKeys' ) )

        new_results = []

        seen={} # eliminate duplicates
        for res in results:
            if not hasattr(res, 'getObject'): continue            
            if seen.has_key( res.getURL() ): continue
            seen[ res.getURL() ]=1
            new_results.append( res )

        return new_results

            
    security.declarePublic( 'getNeoBoardArticles' )                
    def getNeoBoardArticles( self
                    , top=None
                    , expand_all=0
                    , skey=None
                    , reverse=0
                    , default_reverse=0
                    , search_fields=''
                    , search_keywords=''
                    , b_start=0
                    , limit=None ):
        """
            Wrapper to catch catalog error and act on it
        """

        try:
            return self._getNeoBoardArticles( top=top
                    , expand_all=expand_all
                    , skey=skey
                    , reverse=reverse
                    , default_reverse=default_reverse
                    , search_fields=search_fields
                    , search_keywords=search_keywords
                    , b_start=b_start
                    , limit=limit )
        except Exception, e:
            self.np_log(message="Catalog for %s corrupt. Recreating...(%s)'" % ( self.getId(), e ), summary='NeoBoard', severity='ERROR')
            self.initializeNeoPortalElementContainerCatalog()
            return self._getNeoBoardArticles( top=top
                    , expand_all=expand_all
                    , skey=skey
                    , reverse=reverse
                    , default_reverse=default_reverse
                    , search_fields=search_fields
                    , search_keywords=search_keywords
                    , b_start=b_start
                    , limit=limit )            
    
    security.declarePrivate( '_getNeoBoardArticles' )                
    def _getNeoBoardArticles( self
                    , top=None
                    , expand_all=0
                    , skey=None
                    , reverse=0
                    , default_reverse=0
                    , search_fields=''
                    , search_keywords=''
                    , b_start=0
                    , limit=None ):
        """
            Return a complete list of brains objects from the Catalog
        """
        
        # FIXME: temp code to regenerate catalog 
            
        if not hasattr( aq_base( self ), '_catalog'):
            self.initializeNeoPortalElementContainerCatalog()
            
        if search_fields and search_keywords:

            index_map = {'title': 'Title'
                    , 'creator':'Creator'
                    , 'creator_email':'CreatorEmail'
                    , 'body':'getNeoPortalContentSearchText'
                    , 'category':'getNeoBoardArticleCategory'
                    , 'comments':'getInlineCommentsSearchText'
                    , 'comment_creators':'getInlineCommentCreators'
                    , 'attachments': 'getAttachmentsSearchText'}    
                
            new_indexes = []
            for index in search_fields.keys():
                new_indexes.append( index_map[index] )

            results = self.search( indexes=new_indexes, keywords=search_keywords.split( ' ' )  )

        else:
            # threads are automatically reverse-sorted on sort_keys

            reverse = int( reverse )
            result_limit = None
            b_start = int( b_start )
            if limit is not None:
                limit = int( limit )
                result_limit = b_start + limit

            if skey:
                if skey=='np_read_count': skey = 'getNeoPortalReadCount'
                elif skey=='np_num_ratings': skey = 'getNeoPortalNumContentRatings'
                else: skey = 'CreationDate'
                sort_order = ''                
                if reverse: sort_order = 'reverse'
                results = self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, sort_on=skey, sort_order=sort_order, limit = result_limit )
                
            else:
                expand_all = int( expand_all )
                current_thread = None
                if expand_all:
                    results = self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, sort_on='getSortKeys', limit = result_limit )        
                else:
                    if top is not None and top.meta_type == NeoBoardArticle.meta_type:
                        results = self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, isThreadParent=( not expand_all ), sort_on='getSortKey', limit = result_limit )
                        # also pull all the replies to the current article
                        top = top.getNeoBoardArticleThreadParent()
                        sort_key = top.getSortKey()
                        current_thread = self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, getSortKey=sort_key, isThreadParent=0, sort_on='getSortKeys' )
                        idx = 0
                        prefix = self.getNeoPortalElementContainerCatalog().absolute_url()
                        for r in results:
                            if ( prefix + r.getPath() ) == top.absolute_url(): break
                            idx = idx + 1
                        
                        first_half = results[ : idx + 1]
                        second_half = results[ idx + 1: ]
                        results = first_half + current_thread[:] + second_half
                        
                    else:
                        results = self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, isThreadParent=( not expand_all ), sort_on='getSortKey', limit = result_limit )
            
        notices = self.getNotices()
        if len( notices ) > 0: results = notices[:] + results[:]
        return results

    security.declarePublic( 'getNeoBoardArticle' )      
    def getNeoBoardArticle( self, id ):
        """
            Return the named Article
        """     

        return self.getNeoPortalElement( id )
            
    security.declarePublic( 'getNotices' )                
    def getNotices( self):
        """
            Return a list of NeoBoard Notice Articles: articles/notices get concatenated; both should use LazyMap
        """     
        return self.searchResults( meta_type=NeoBoardNoticeArticle.meta_type, isTempNeoBoardArticle=0, sort_on='getSortKeys', limit=10 )
                
    security.declarePublic( 'getCurrentArticleURL' )
    def getCurrentArticleURL( self):
        """
            Return the relative URL of the current article
        """     
        
        url = self.REQUEST.URL1
        return url.replace( self.getNeoBoard().absolute_url() + '/', '' )        
    
    security.declarePublic( 'getPostingCategories' )    
    def getPostingCategories( self):
        """
            Return Posting Categories
        """             
        return self.posting_categories

    security.declarePublic( 'getPostingCategoryKeys' )    
    def getPostingCategoryKeys( self ):
        """
            Return Posting Category Keys
        """             
        return self.getPostingCategories().keys()
        
    security.declarePublic( 'getPostingCategoryName' )    
    def getPostingCategoryName( self, key):
        """
            Return Posting Category Name
        """
        try:
            return self.getPostingCategories()[key]
        except:
            return ''

    security.declarePublic( 'getNeoBoardManagers' )                
    def getNeoBoardManagers( self ):
        """
            Returns Board Managers
        """     
        return self.managers        

    security.declarePublic( 'getNeoBoardManagerKeys' )                
    def getNeoBoardManagerKeys( self ):
        """
            Returns Board Manager Keys
        """     
        return self.getNeoBoardManagers().keys()
        
    security.declarePublic( 'getNeoBoardManagerName' )    
    def getNeoBoardManagerEmail( self, key ):
        """
            Return NeoBoard Manager Email
        """
        try:
            return self.getNeoBoardManagers()[key]
        except:
            return ''        

    security.declarePublic( 'getNextArticleNum' )
    def getNextArticleNum( self, notice=0):
        """
            Return the next article number to use
        """     

        if notice:
            return self.resetElementNumbers( e_type = NeoBoardNoticeArticle.meta_type )
        else:
            return self.resetElementNumbers( e_type = NeoBoardArticle.meta_type )

    security.declarePublic( 'getTotalNumArticles' )
    def getTotalNumArticles( self):
        """
            Return the total number of articles this board contains
        """     
        
        return len( self.searchResults( meta_type=NeoBoardArticle.meta_type ) )

    security.declarePublic( 'useFileSystem' )
    def useFileSystem( self):
        """
            Tell if attachments are saved on the filesystem
        """     
        
        if hasattr( self, 'use_filesystem' ):
            return self.use_filesystem
        else:
            return 0

    security.declarePublic( 'getArchivePath' )
    def getArchivePath( self):
        """
            Return Archive Path
        """     
        
        return self.archive_path

    security.declareProtected( NeoPortalManageProperties, 'setArchivePath' )
    def setArchivePath( self, archive_path):
        """
            Return Archive Path
        """    
        self.archive_path = archive_path

    security.declarePublic( 'getThreadDepthHspace' )
    def getThreadDepthHspace( self ):
        """
            Return Thread Depth Hspace
        """    
        
        if not hasattr( aq_base( self ), 'thread_depth_hspace' ):
            self.thread_depth_hspace=5

        return self.thread_depth_hspace

    security.declarePublic( 'getMaxThreadDepth' )
    def getMaxThreadDepth( self ):
        """
            Return Maximum Thread Depth
        """    
        
        if not hasattr( aq_base( self ), 'max_thread_depth' ):
            self.max_thread_depth=5

        return self.max_thread_depth        

    security.declarePublic( 'getBadExtensions' )
    def getBadExtensions( self):
        """
            Filters out bad extensions
        """
        extensions = []

        for ext in self.bad_extensions:
            if not ext.startswith( '.' ):
                extensions.append( '.' + ext )
            else:
                extensions.append( ext )
        return extensions       


    security.declarePublic( 'getMaxFileSize' )
    def getMaxFileSize( self):
        """
            Return the Upper Size Limit to Attachments
        """     
        return self.max_file_size

    security.declarePublic( 'getMaxImageWidth' )
    def getMaxImageWidth( self):
        """
            Return Maximum Image Width to trigger a thumbnail
        """     
        return self.max_image_width
    
    security.declarePublic( 'getMaxImageHeight' )
    def getMaxImageHeight( self):
        """
            Return Maximum Image Height to trigger a thumbnail
        """             
        return self.max_image_height

    security.declarePublic( 'getMaxListTitleLength' )
    def getMaxListTitleLength( self):
        """
            Return maximum length of titles when appearing on article lists
        """
        return self.max_list_title_length

    security.declarePublic( 'isMailEnabled' )
    def isMailEnabled( self):
        """
            Tell if mailing is enabled
        """     
        
        return self.mail_article

    security.declarePublic( 'isReadOnly' )
    def isReadOnly( self):
        """
            Tell if the board is read-only
        """     
    
        return self.read_only

    security.declarePublic( 'attachmentsAllowed' )
    def attachmentsAllowed( self):
        """
            Tell if attachments are allowed
        """     

        return self.getNumAttachmentsAllowed() > 0

    security.declarePublic( 'getNumAttachmentsAllowed' )
    def getNumAttachmentsAllowed( self):
        """
            Return the number of attachments allowed
        """     
        
        num = self.num_attachments
        
        if num > self._max_num_attachments:
            return self._max_num_attachments
        else:
            return num
            
    security.declarePublic( 'emptyBodyAllowed' )
    def emptyBodyAllowed( self):
        """
            Tell if empty body is allowed
        """     
        
        return self.allow_empty_body
        
    security.declarePublic( 'useACLUserAuth' )
    def useACLUserAuth( self):
        """
            Tell if ACL user authentication is in use
        """     
        
        if hasattr( self, 'acl_user_auth' ):
            return self.acl_user_auth
        else:
            return 0

    security.declarePublic( 'anonymousPostingAllowed' )
    def anonymousPostingAllowed( self):
        """
            Tell if anonymous posting is allowed
        """     
        
        if hasattr( self, 'anonymous_posting_allowed' ):
            return self.anonymous_posting_allowed
        else:
            return 0

    security.declarePublic( 'emptyPasswordAllowed' )
    def emptyPasswordAllowed( self):
        """
            Tell if empty password is allowed
        """     
        
        if hasattr( self, 'allow_empty_password' ):
            return self.allow_empty_password
        else:
            return 0

    security.declarePublic( 'emptyEmailAllowed' )
    def emptyEmailAllowed( self):
        """
            Tell if empty email address is allowed
        """     
        
        if hasattr( self, 'allow_empty_email' ):
            return self.allow_empty_email
        else:
            return 0
    
    security.declarePublic( 'autoApproveArticle' )
    def autoApproveArticle( self):
        if hasattr( self, 'auto_approve_article' ):
            return self.auto_approve_article
        else:
            return 0

    security.declarePublic( 'isNeoBoardManager' )
    def isNeoBoardManager( self, REQUEST=None ):
        """
            Tell if the user is a manager
        """

        auth_user = self.getAuthenticatedNeoPortalUser()
        if auth_user is None:
            return 0
 
        if self.isNeoPortalContentOwner():
            return 1
 
        if auth_user.getUserName() in self.getNeoBoardManagers().keys():
            return 1
            
        if auth_user.has_permission( NeoPortalManagePortal, self.getNeoBoard()  ):
            return 1

        return 0

    security.declarePublic( 'getMailHosts' )
    def getMailHosts( self):
        """
            Return the list of available mailhost objects
        """ 
        try:
            return self.superValues(( 'Mail Host',) )
        except:
            return []

    security.declarePublic( 'getMailHost' )
    def getMailHost( self):
        """ 
            Return the current mailhost instance
        """
        try:
            return getattr( self, self.mailhost)
        except AttributeError:
            return None

    security.declarePrivate( 'sendmail' )
    def sendmail( self, from_addr, to_addr, reply_addr, subject, body, attachments=None, html=0 ):
        """ 
            Sends out emails
        """ 
        mailhost = self.getMailHost()
        if mailhost:
            mailer = NeoPortalMailer( mailhost=mailhost 
                                    , from_addr=from_addr
                                    , to_addr=to_addr
                                    , reply_addr=reply_addr
                                    , subject = subject
                                    , body = body
                                    , attachments = attachments
                                    , charset=self.getNeoBoardCharset()
                                    , html = html)
            mailer.send()
        else:
            raise 'NeoBoardError', "Mailhost not set!"
  
    security.declarePrivate( '_reRenderAllNeoBoardArticles' )
    def _reRenderAllNeoBoardArticles( self):
        """ 
            Re-renders all renderable objects the container holds when renderable format gets changed
        """
        
        dummy = [ brain.getObject()._updateNeoPortalContent() for brain in self.getNeoBoardArticles() if not hasattr( brain.getObject(), '_is_temp_article' ) ]

    def __getattr__( self, name ):
        """
            Search NeoBoard Skin Folder for 'name'
            
            CMFCore's Skinnable.py adapted
        """
                        
        if not name.startswith( '_' ) and not name.startswith( 'aq_' ):            
            sd = self._v_neoboard_skindata
            if sd is not None:
                request,skin, ignore = sd

                if not ignore.has_key( name ):
                    elem = skin._getSkinElement( name )
                    if elem is not None:
                            
                        return aq_base( elem )
                    else:
                        ignore[name] = 1
                else:
                    pass
                
        if superGetAttr is None:    
            raise AttributeError, name

    security.declarePrivate( 'setupCurrentSkin' )
    def setupCurrentSkin( self, REQUEST=None ):
        """
            Set up _v_neoboard_skindata 
            
            CMFCore's Skinnable.py adapted
        """        

        if REQUEST is None:
            REQUEST = getattr( self, 'REQUEST', None )

        if REQUEST is None:
            return
            
        if self._v_neoboard_skindata is not None and self._v_neoboard_skindata[0] is REQUEST:        
            return

        # set up the current NeoBoard Skin

        try:
            self._v_neoboard_skindata = ( REQUEST, self.getNeoBoardSkin(), {} )
        except:
            pass        

    def __of__( self, parent ):
        """
            Set up NeoBoard Skin
            
            CMFCore's Skinnable.py adapted
        """              

        w_self = ImplicitAcquisitionWrapper( aq_base( self ), parent )
        w_self.setupCurrentSkin()
        
        return w_self

    security.declarePrivate( '_checkId' )
    def _checkId( self, id, allow_dup=0):
        """
            Allow objects with Ids that match the Id of a skin element.
            
            CMFCore's Skinnable.py adapted
        """ 
        
        if not allow_dup:
            sd = self._v_neoboard_skindata
            self._v_neoboard_skindata = None
            
            try:
                base = getattr( self, 'aq_base', self )
                if not hasattr(base, id):
                    return
            finally:
                self._v_neoboard_skindata = sd
        
    security.declareProtected( NeoPortalManageProperties, 'manage_editNeoBoard' )
    def manage_editNeoBoard( self
                            , title=''
                            , description=''
                            , email_from_name=''
                            , email_from_address=''                            
                            , neoboard_skin='default'
                            , mailhost=''
                            , allow_inline_comments=0
                            , max_inline_comment_size=1000
                            , allow_content_iconization = 0
                            , spam_blocking_interval = 10
                            , enable_spam_blocking = 0
                            , allow_content_rating=0
                            , max_content_rating_value=10
                            , min_content_rating_value=1
                            , access_log_access_trace_key=[]
                            , content_rating_access_trace_key=[]
                            , use_filesystem=0
                            , archive_path=''
                            , bad_extensions=[]
                            , max_file_size=5
                            , max_image_width=200
                            , max_image_height=200
                            , max_list_title_length=30
                            , items_per_page=20
                            , max_thread_depth=5
                            , thread_depth_hspace=5
                            , format='HTML'
                            , charset=''
                            , language=''
                            , effective_date=None
                            , expiration_date=None
                            , enabled=0
                            , read_only=0
                            , mail_article=0
                            , acl_user_auth=0
                            , num_attachments=0
                            , allow_empty_body=0
                            , allow_empty_password=0
                            , allow_empty_email=0
                            , auto_approve_article=0
                            , allow_anonymous_posting=0
                            , recreate_catalog=0
                            , check_filepath=1
                            , REQUEST=None ):
        """ 
            ZMI edit method
        """

        result = self._edit( title=title
                    , description=description
                    , email_from_name=email_from_name
                    , email_from_address=email_from_address          
                    , neoboard_skin=neoboard_skin
                    , mailhost=mailhost
                    , allow_inline_comments=allow_inline_comments
                    , max_inline_comment_size=max_inline_comment_size
                    , allow_content_iconization = allow_content_iconization
                    , spam_blocking_interval = spam_blocking_interval
                    , enable_spam_blocking = enable_spam_blocking
                    , allow_content_rating=allow_content_rating
                    , max_content_rating_value=max_content_rating_value
                    , min_content_rating_value=min_content_rating_value                    
                    , access_log_access_trace_key=access_log_access_trace_key
                    , content_rating_access_trace_key=content_rating_access_trace_key
                    , use_filesystem=use_filesystem
                    , archive_path=archive_path
                    , bad_extensions=bad_extensions
                    , max_file_size=max_file_size
                    , max_image_width=max_image_width
                    , max_image_height=max_image_height
                    , max_list_title_length=max_list_title_length
                    , items_per_page=items_per_page
                    , max_thread_depth=max_thread_depth
                    , thread_depth_hspace=thread_depth_hspace
                    , format=format
                    , charset=charset
                    , language=language
                    , effective_date=effective_date
                    , expiration_date=expiration_date
                    , enabled=enabled
                    , read_only=read_only
                    , mail_article=mail_article
                    , acl_user_auth=acl_user_auth
                    , num_attachments=num_attachments
                    , allow_empty_body=allow_empty_body
                    , allow_empty_password=allow_empty_password
                    , allow_empty_email=allow_empty_email
                    , auto_approve_article=auto_approve_article
                    , allow_anonymous_posting=allow_anonymous_posting
                    , recreate_catalog=recreate_catalog
                    , check_filepath=check_filepath
                    , REQUEST=REQUEST )    
        
        if result is not None:
            return result

        if REQUEST is not None:
            return apply( self.neoboard_main, ( self, self.manageNeoPortalUserMessage( REQUEST, message='Changes applied!' ) ) )            

    security.declareProtected( NeoPortalManageProperties, 'edit' )
    def edit( self
            , title=''
            , description=''
            , email_from_name=''
            , email_from_address=''            
            , neoboard_skin='default'
            , mailhost=''
            , allow_inline_comments=0
            , max_inline_comment_size=1000
            , allow_content_iconization = 0
            , spam_blocking_interval = 10
            , enable_spam_blocking = 0
            , allow_content_rating=0
            , max_content_rating_value=10
            , min_content_rating_value=1
            , access_log_access_trace_key=[]
            , content_rating_access_trace_key=[]
            , use_filesystem=0
            , archive_path=''
            , bad_extensions=[]
            , max_file_size=5
            , max_image_width=200
            , max_image_height=200
            , max_list_title_length=30
            , items_per_page=20
            , max_thread_depth=5
            , thread_depth_hspace=5
            , format='HTML'
            , charset=''
            , language=''
            , effective_date=None
            , expiration_date=None
            , enabled=0
            , read_only=0
            , mail_article=0
            , acl_user_auth=0
            , num_attachments=0
            , allow_empty_body=0
            , allow_empty_password=0
            , allow_empty_email=0
            , auto_approve_article=0
            , recreate_catalog=0
            , check_filepath=1
            , REQUEST=None ):
        """
            edit
        """
        self._edit( title=title
                    , description=description
                    , email_from_name=email_from_name
                    , email_from_address=email_from_address          
                    , neoboard_skin=neoboard_skin
                    , mailhost=mailhost
                    , allow_inline_comments=allow_inline_comments
                    , max_inline_comment_size=max_inline_comment_size
                    , allow_content_iconization = allow_content_iconization
                    , spam_blocking_interval = spam_blocking_interval
                    , enable_spam_blocking = enable_spam_blocking                    
                    , allow_content_rating=allow_content_rating
                    , max_content_rating_value=max_content_rating_value
                    , min_content_rating_value=min_content_rating_value                    
                    , access_log_access_trace_key=access_log_access_trace_key
                    , content_rating_access_trace_key=content_rating_access_trace_key
                    , use_filesystem=use_filesystem
                    , archive_path=archive_path
                    , bad_extensions=bad_extensions
                    , max_file_size=max_file_size
                    , max_image_width=max_image_width
                    , max_image_height=max_image_height
                    , max_list_title_length=max_list_title_length
                    , items_per_page=items_per_page
                    , max_thread_depth=max_thread_depth
                    , thread_depth_hspace=thread_depth_hspace
                    , format=format
                    , charset=charset
                    , language=language
                    , effective_date=effective_date
                    , expiration_date=expiration_date
                    , enabled=enabled
                    , read_only=read_only
                    , mail_article=mail_article
                    , acl_user_auth=acl_user_auth
                    , allow_anonymous_posting=allow_anonymous_posting
                    , num_attachments=num_attachments
                    , allow_empty_body=allow_empty_body
                    , allow_empty_password=allow_empty_password
                    , allow_empty_email=allow_empty_email
                    , auto_approve_article=auto_approve_article
                    , recreate_catalog=recreate_catalog
                    , check_filepath=check_filepath
                    , REQUEST=REQUEST )
                    
    security.declarePrivate( '_edit' )
    def _edit( self
                , title=''
                , description=''
                , email_from_name=''
                , email_from_address=''
                , neoboard_skin='default'
                , mailhost=''
                , allow_inline_comments=0
                , max_inline_comment_size=1000
                , allow_content_iconization = 0
                , spam_blocking_interval = 10
                , enable_spam_blocking = 0
                , allow_content_rating=0
                , max_content_rating_value=10
                , min_content_rating_value=1
                , access_log_access_trace_key=[]
                , content_rating_access_trace_key=[]
                , use_filesystem=0
                , archive_path=''
                , bad_extensions=[]
                , max_file_size=5
                , max_image_width=200
                , max_image_height=200
                , max_list_title_length=30
                , items_per_page=20
                , max_thread_depth=5
                , thread_depth_hspace=5
                , format='STX'
                , charset=''
                , language=''
                , effective_date=None
                , expiration_date=None
                , enabled=0
                , read_only=0
                , mail_article=0
                , acl_user_auth=0
                , allow_anonymous_posting=0
                , num_attachments=0
                , allow_empty_body=0
                , allow_empty_password=0
                , allow_empty_email=0
                , auto_approve_article=0    
                , recreate_catalog=0
                , check_filepath=1
                , REQUEST=None ):
        """
            Actual editing occurs here
        """             
        
        re_render_articles = 0
        
        self.setTitle( title )
        self.setDescription( description )
        self.setNeoBoardSkin( neoboard_skin )

        if email_from_name and email_from_address:
            if not isEmailValid( email_from_address ):
                return self.manageNeoPortalError( REQUEST=REQUEST, field='email_from_address', message="Invalid Email Address: %s" % email_from_address )
            else:
                self.email_from_name = email_from_name
                self.email_from_address = email_from_address
        
        if mailhost:
            try:    
                dummy=getattr( self, mailhost)
            except AttributeError: 
                return self.manageNeoPortalError( REQUEST=REQUEST, field='mailhost', message="Invalid Mailhost!" )
        
            self.mailhost = mailhost
        
        if use_filesystem and not archive_path:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message="You must specify the archive path to save attachments on the filesystem!" )

        if use_filesystem and archive_path and not num_attachments:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='num_attachments', message="You must allow at least one attachment to save attachments on the filesystem!" )

            
        if ( num_attachments > 0 and use_filesystem ) or self.isUsingTempArchive():
            if not archive_path:
                return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message="You must specify the archive path to allow attachments!" )
            else:
            
                if check_filepath and not checkFilePath( archive_path ):
                    return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message="Unauthorized archive path!" )
                    
                if not archive_path.endswith( os.sep ):
                    archive_path = archive_path + os.sep

                if self.isUsingTempArchive():
                    # temporary archive? must have been moved or copied. rename it.                    
                    
                    os.rename( self.getArchivePath(), archive_path )
                    res = _mark_archive_path( archive_path, self.absolute_url() )                    
                    delattr( self,  '_temp_archive_set' )
                    self.setArchivePath(archive_path)
                    # call every attachment to _resetFilePath
                    
                    dummy = [ attachment.getObject()._resetFilePath() for attachment in self.getNeoPortalElements( e_type= NeoBoardArticleAttachment.meta_type ) if attachment.getObject().isNeoPortalExternalFile() ]
                        
                    use_filesystem = 1
                else:
                    if os.path.isdir( archive_path ):
                        if not os.access( archive_path, os.R_OK | os.W_OK ):
                            return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message="You don't have permissions to use that archive path!" )
                        
                        res = _mark_archive_path( archive_path, self.absolute_url() )
                        if res != 1:
                            return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message=res )
                            
                    else:
                        try:
                            os.mkdir( archive_path, 0755 )
                            res = _mark_archive_path( archive_path, self.absolute_url() )
                            if res != 1:
                                return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message=res )
                        except:
                            return self.manageNeoPortalError( REQUEST=REQUEST, field='archive_path', message="Archive path doesn't exist and cannot be created!" )

            self.setArchivePath( archive_path )
        else:
            self.setArchivePath( '' )
        
        self.use_filesystem=use_filesystem
        self.bad_extensions=bad_extensions
        self.num_attachments=num_attachments
        self.items_per_page = items_per_page

        self.thread_depth_hspace=thread_depth_hspace
        if self.thread_depth_hspace > self._max_thread_depth_hspace:
            self.thread_depth_hspace=self._max_thread_depth_hspace

        self.max_thread_depth=max_thread_depth
        if self.max_thread_depth < 0:
            self.max_thread_depth=5
            
        self.setNeoBoardEnabled( mode=enabled )
        self.read_only=read_only
    
        if not self.getTotalNumArticles():
            self.acl_user_auth = acl_user_auth
            self.use_filesystem = use_filesystem

        self.anonymous_posting_allowed = allow_anonymous_posting

        self.max_file_size = max_file_size
        self.max_image_width=max_image_width
        self.max_image_height=max_image_height
        self.max_list_title_length=max_list_title_length
        self.mail_article=mail_article
        self.allow_empty_body = allow_empty_body
        self.auto_approve_article = auto_approve_article

        if not self.useACLUserAuth():
            self.allow_empty_password = allow_empty_password
            self.allow_empty_email = allow_empty_email

        if format and hasattr( self, 'format' ) and format != self.Format(): 
            self.format = format
            re_render_articles = 1

        if charset: 
            self.neoboard_charset=charset
        else:
            self.neoboard_charset=self.getDefaultNeoBoardCharset()
    
        if language:        
            self.language=language
        else:
            self.language=self.getDefaultNeoBoardLanguage()
        
        if effective_date:
            try:
                dt = DateTime(effective_date)
                self.effective_date = dt
            except:    
                pass
        else:
            self.effective_date = DateTime()

        if expiration_date:
            try:
                dt = DateTime(expiration_date)
                self.expiration_date = dt
            except:    
                pass
        else:
            self.expiration_date = None
            
        #############################################################
        # NeoPortal Content Options Begin
        #
        self.np_spam_blocking_enabled = enable_spam_blocking 
        self.np_spam_blocking_interval = spam_blocking_interval 
        self.np_max_inline_comment_size = max_inline_comment_size
        self.np_content_rating_allowed = allow_content_rating
        self.np_max_content_rating_value = max_content_rating_value 
        self.np_min_content_rating_value = min_content_rating_value 

        if hasattr( aq_base( self ), 'np_content_iconization_allowed') and self.np_content_iconization_allowed != allow_content_iconization:
            re_render_articles = 1  

        if hasattr( aq_base( self ), 'np_inline_comments_allowed') and self.np_inline_comments_allowed != allow_inline_comments:
            re_render_articles = 1  
            
        self.np_content_iconization_allowed = allow_content_iconization
        self.np_inline_comments_allowed = allow_inline_comments
        
        if access_log_access_trace_key and 'Date' in access_log_access_trace_key and access_log_access_trace_key != self.getNeoPortalAccessTraceKey( container_name=self._np_access_log_container_name ):
            self.setNeoPortalAccessTraceKey( access_trace_key=access_log_access_trace_key, container_name=self._np_access_log_container_name )

        if content_rating_access_trace_key and ( 'IP Address' in content_rating_access_trace_key  or 'Username' in content_rating_access_trace_key )and content_rating_access_trace_key != self.getNeoPortalAccessTraceKey( container_name=self._np_content_rating_container_name ):
            self.setNeoPortalAccessTraceKey( access_trace_key=content_rating_access_trace_key, container_name=self._np_content_rating_container_name )            

        #
        # NeoPortal Content Options End
        #############################################################

        if re_render_articles:
            self._reRenderAllNeoBoardArticles()

        if recreate_catalog:
            self._initNeoPortalElementContainerCatalog()
                       
        try:
            catalog = getToolByName( self, 'portal_catalog', None )
            if catalog is not None:
                catalog.unindexObject( self )
                catalog.indexObject( self )
        except:
            pass            
            
    security.declareProtected( NeoPortalManageProperties, 'getAllowedFileBases' )
    def getAllowedFileBases( self ):
        """
            Return allowed file bases
        """

        return getAllowedFileBases()            
            
    security.declareProtected( NeoPortalManageProperties, 'manage_batchAction' )
    def manage_batchAction( self,
            ids=None,
            submit=None,
            to_board=None,
            REQUEST=None ):
        """
            Batch Action
        """

        if ids is None or len( ids ) <= 0:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="No items selected!" )

        if submit is None:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="Invalid operation!" )

        submit = submit.lower()

        message = ''

        if submit == 'move' or submit == 'copy':
            if to_board is None:
                return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="Receiving board not specified!" )
                
            if self.useACLUserAuth() != self.aq_inner.aq_parent._getOb(to_board).useACLUserAuth():
                return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="The receiving board is using a different authentication scheme." )

            if self.useFileSystem() != self.aq_inner.aq_parent._getOb(to_board).useFileSystem():
                return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="The receiving board is using a different attachment storage." )

            for article in ids:
                article_obj = self._getOb( article )
                if self.aq_parent._getOb( to_board ).getNumAttachmentsAllowed() < article_obj.getNumAttachments():
                    return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="The receiving board has can't hold all the attachments the article '%s' has.", tup= (article_obj.title_or_id(),) )
            try:
                receiver = self.aq_inner.aq_parent._getOb( to_board )
            except AttributeError:
                return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="The named receiving board doesn't exist." )

            if submit == 'move':
                pointer = self.manage_cutObjects( ids )
                message = self.getNeoPortalMessage( message="%d thread(s) moved to '%s' board.", tup=( len( ids ), receiver.title_or_id() ) )
            else:
                pointer = self.manage_copyObjects( ids )
                message = self.getNeoPortalMessage( message="%d thread(s) copied to '%s' board.", tup=( len( ids ), receiver.title_or_id() ) )
                    
            results = receiver.manage_pasteObjects( pointer )

            objs = [ receiver._getOb( dict['new_id'] ) for dict in results ]            

        elif submit == 'approve' or submit == 'disapprove':
            threads = 0
            for item in ids:
                try:
                    if submit=='approve':
                        self._getOb( item ).approveThread()
                    else:
                        self._getOb( item ).disApproveThread()
                        
                    threads = threads + 1
                except:
                    threads = threads - 1
            
            if submit=='approve':
                message=self.getNeoPortalMessage( message="%d thread(s) approved out of %d thread(s) requested.", tup=( threads, len(ids) ) )
            else:
                message=self.getNeoPortalMessage( message="%d thread(s) disapproved out of %d thread(s) requested.", tup=( threads, len(ids) ) )
                
        elif submit == 'delete':
            num_messages = len(ids)
            self.manage_delObjects(ids)
            message=self.getNeoPortalMessage( message=( "%d thread(s) deleted." ), tup=( num_messages, ) )
                
        if REQUEST is not None:
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )            

    security.declareProtected( NeoPortalManageProperties, 'manage_addPostingCategory' )
    def manage_addPostingCategory( self, category_id, category_title, REQUEST=None ):
        """
            Add Posting Category
        """

        if not category_id:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='category_id', message="The field cannot be empty!" )

        if not category_title:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='category_title', message="The field cannot be empty!" )    
            
        if not hasattr( aq_base( self ), 'posting_categories' ) or not self.posting_categories:
            self.posting_categories = PersistentMapping()

        self.posting_categories[category_id] = category_title

        if REQUEST is not None:
            REQUEST['category_id'] = ''
            REQUEST['category_title'] = ''
        
            return apply( self.neoboard_main, ( self, REQUEST, 1 ) )

    security.declareProtected( NeoPortalManageProperties, 'manage_editPostingCategory' )
    def manage_editPostingCategory( self, category_id, new_category_id, category_title, REQUEST=None ):
        """
            Edit Posting Category
        """

        if not new_category_id:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='new_category_id', message="The field cannot be empty!" )

        if not category_title:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='category_title', message="The field cannot be empty!" )
            
        if category_id != new_category_id:
            results = self.getNeoPortalElements( e_type = NeoBoardArticle.meta_type )
            for result in results:
                article = result.getObject()
                if article.getNeoBoardArticleCategory() == category_id:
                    article.setNeoBoardArticleCategory( new_category_id )
            
            if self.posting_categories.has_key( category_id ):
                del self.posting_categories[category_id]
                self.posting_categories[new_category_id] = category_title               
        else:
            self.posting_categories[category_id] = category_title

        if REQUEST is not None:
            REQUEST['category_id'] = ''
            REQUEST['category_title'] = ''
        
            return apply( self.neoboard_main, ( self, REQUEST, 1 ) )

    security.declareProtected( NeoPortalManageProperties, 'manage_delPostingCategory' )
    def manage_delPostingCategory( self, category_id, REQUEST=None ):
        """
            Delete Posting Category
        """

        if not category_id:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='category_id', message="The field cannot be empty!" )

        if self.posting_categories.has_key( category_id ):
            del self.posting_categories[category_id]

        if REQUEST is not None:
            return apply( self.neoboard_main, ( self, REQUEST, 1 ) )
    

    security.declareProtected( NeoPortalManageProperties, 'manage_addNeoBoardManager' )
    def manage_addNeoBoardManager( self, username, email=None, REQUEST=None ):
        """
            Add Manager
        """

        if not username:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='username', message="The field cannot be empty!" )

        user = self.acl_users.getUser( username )
        
        if not user:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='username', message="No Such User!" )
        
        if not email:
            email = None
            
        if not hasattr( aq_base( self ), 'managers' ) or not self.managers:
            self.managers = PersistentMapping()
            
        self.managers[username] = email

        if REQUEST is not None:
            REQUEST['username'] = ''
            REQUEST['email'] = ''
            return apply( self.neoboard_main, ( self, REQUEST, 1 ) )

    
    manage_editNeoBoardManager = manage_addNeoBoardManager

    security.declareProtected( NeoPortalManageProperties, 'manage_delNeoBoardManager' )
    def manage_delNeoBoardManager( self, username, REQUEST=None ):
        """
            Delete Manager
        """

        if not username:
            return self.manageNeoPortalError( REQUEST=REQUEST, field='username', message="The field cannot be empty!" )

        if self.managers.has_key( username ):
            del self.managers[username]

        if REQUEST is not None:
            return apply( self.neoboard_main, ( self, REQUEST, 1 ) )
                
    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd( self, item, container):
        """
            Tell if self is being copied
        """             

        if self.useFileSystem():
            # When adding a new NeoBoard, use_filesystem is not set to true, not matter what,
            # at the time of its instantiation.
            # If it's set, that means this instance is being copied or moved!
        
            if self.archive_path.endswith( os.sep ):
                self.archive_path = self.archive_path[:-1]
                
            self.archive_path =  self.archive_path + '_TEMP_ARCHIVE_' + self.getId() + os.sep
            self._temp_archive_set = 1
    
            os.mkdir( self.archive_path, 0755 )
                            
        return NeoBoard.inheritedAttribute("manage_afterAdd" )( self, item, container )  

    security.declareProtected( NeoPortalManageProperties, 'isUsingTempArchive' )
    def isUsingTempArchive( self):
        """
            Tell if this NeoBoard recently has been copied/moved and using a temporary archive path?
        """
        return hasattr( self, '_temp_archive_set' )
            
    ###################################################################################
    # 
    #       Upgrade methods
    #
    ###################################################################################

    security.declareProtected( NeoPortalManagePortal, 'selfUpgradeNeoPortalContent' )
    def selfUpgradeNeoPortalContent( self, force_upgrade=0):
        """
            Upgrade Self
        """

        if not hasattr( self, '_np_upgraded_up_to') or ( self._np_upgraded_up_to != __NEOPORTAL_CLASS_VERSION__ and self._np_upgraded_up_to != __PREVIOUS_NEOPORTAL_CLASS_VERSION__ ):
            return "You should upgrade the object <em>%s</em> of type <em>%s</em> to version %s first. Read the UPGRADE.txt for more info." % ( self.getId(), self.meta_type, __PREVIOUS_NEOPORTAL_CLASS_VERSION__ )

        if force_upgrade or self._np_upgraded_up_to != __NEOPORTAL_CLASS_VERSION__:
            # we don't need the neoboard catalog anymore
            
            try:
                 self.manage_delObjects( [ 'NeoBoardCatalog_1' ] )
            except:
                pass
            
            # initialize built-in catalog
            
            self.initializeNeoPortalElementContainerCatalog()

            self.setNeoPortalAccessTraceKey( access_trace_key=['Date',], container_name=self._np_access_log_container_name )
            self.setNeoPortalAccessTraceKey( access_trace_key=['IP Address','Username'], container_name=self._np_content_rating_container_name )            

            
            articles = [ obj for id, obj in self.ZopeFind( self, obj_metatypes = [ NeoBoardArticle.meta_type, NeoBoardNoticeArticle.meta_type ], search_sub=1 ) ]
            for article in articles:
                article._setSortKey()
                article._setThreadSortKey()
                article._updateNeoPortalContent()
                self.updateNeoPortalElementContainerCatalogIndex( article )
        
            self.notifyNeoPortalObjectUpgradeComplete()
            return "Upgrade complete: <b>%s</b>. Click <a href=\"%s\"><b>here</b></a> to get back to your upgraded board." % ( self.meta_type, self.absolute_url() )
        else:
            return "<em><b> %s </b></em> of <em> %s </em> class is up to date." % ( self.getId(), self.meta_type )
        
InitializeClass( NeoBoard )
