#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardArticle.py,v 1.32 2003/04/03 16:40:24 wankyu Exp $
Description

    NeoBoard Article - folderish object that can hold attachments and comments    
"""
  
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from DateTime.DateTime import DateTime
from Acquisition import aq_parent, aq_base, aq_inner

from Products.NeoPortalLibrary.NP_Utils import cutString, isEmailValid, removeTags, encryptNeoPortalPassword

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from Products.NeoBoard.NeoBoardBase import NeoBoardArticleBase
from Products.NeoBoard.NeoBoardArticleAttachment import NeoBoardArticleAttachment, addNeoBoardArticleAttachment
from Products.NeoBoard.NeoBoardArticleComment import addNeoBoardArticleComment, NeoBoardArticleComment, NEOBOARD_REQUEST_COMMENT_PREFIX
from DocumentTemplate.DT_Util import html_quote
import os, mimetypes, re

from Products.NeoBoard.NeoBoard import __NEOPORTAL_CLASS_VERSION__

class NeoBoardArticle( NeoBoardArticleBase ):
    """
        NeoBoard Article
    """
    
    __NEOPORTAL_CLASS_VERSION__  = __NEOPORTAL_CLASS_VERSION__
    
    neoboard_article = 1
    
    neoportal_element_prefix = 'a_' # should end with '_' and should not contain any more '_'    

    neoportal_cacheable_content = 1
    neoportal_inline_commentable_content = 1
    neoportal_spam_blockable_content = 1
    neoportal_iconizable_content = 1
    neoportal_ratable_content = 1
    neoportal_discussable_content = 0
    
    np_access_logging_enabled = 1
    
    meta_type='NeoBoard Article'
    
    meta_types=(
        {'name': 'NeoBoard Article Attachment Element', 'action':'', 'permission': ''},
        {'name': 'NeoBoard Article', 'action':'', 'permission': ''},
    )
    
    all_meta_types=meta_types

    
    icon='misc_/NeoBoard/neoboard_article_icon'
    
    security = ClassSecurityInfo()
    
    def __init__( self, id, password='', title='', text='' ):
        NeoBoardArticleBase.__init__( self, id=id )

        self.password=''
        self.setTitle( title )
        self.setRawNeoPortalContent( text )
        self._setCurrentClassVersion( __NEOPORTAL_CLASS_VERSION__ )

    def __call__( self, REQUEST=None ):
        """
            Default View
        """
        if REQUEST is None:
            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
            else:
                return ''
       
        self.upNeoPortalReadCount()
       
        view = self.neoboard_main
        if getattr( aq_base(view), 'isDocTemp', 0 ):
            return apply( view, ( self, self._cookREQUEST( REQUEST ) ) )
        else:
            return view()

    index_html = __call__        
    
    security.declarePrivate( '_updateNeoPortalContent' )
    def _updateNeoPortalContent( self ):
        """
            Update Content after adding/edit comments
        """

        self.cooked_text = self._renderNeoPortalContent()

        self.getNeoBoard().updateNeoPortalElementContainerCatalogIndex( self )
        

    security.declareProtected( NeoPortalModifyPortalContent, 'getNeoPortalAccessTraceKey' )        
    def getNeoPortalAccessTraceKey( self, container_name='default' ):
        """
            Return how to trace a user access
        """

        neoboard = self.getNeoBoard()

        if not hasattr( aq_base( neoboard ),  '_np_access_traces_containers' ):
            neoboard.initAllNeoPortalAccessTraceContainers()

        if not hasattr( aq_base( self ),  '_np_access_traces_containers' ):
            self.initAllNeoPortalAccessTraceContainers()

        try:
            container, access_trace_key = neoboard._np_access_traces_containers[ container_name ]
        except KeyError:
            neoboard.initNeoPortalAccessTraceContainer( container_name )
            container, access_trace_key = neoboard._np_access_traces_containers[ container_name ]

        if not access_trace_key:
            return neoboard.neoportal_access_trace_keys

        return access_trace_key        

    security.declarePublic( 'isNeoBoardArticle' )
    def isNeoBoardArticle( self ):
        """
            Tell if self is an article
        """
        return self.meta_type == NeoBoardArticle.meta_type

    security.declarePublic( 'checkNeoPortalContentReviewStatus' )
    def checkNeoPortalContentReviewStatus( self ):
        """
            Check content review status: always return true
        """        
        
        return 1
    
    security.declarePublic( 'getNeoBoardArticleURL' )
    def getNeoBoardArticleURL( self ):
        """
            Return relative URL of this article
        """             
        url = self.absolute_url()
        return url.replace( self.getNeoBoardURL() + '/', '' )

    security.declarePublic( 'getNeoBoardArticleDepth' )
    def getNeoBoardArticleDepth( self ):
        """
            Return how deep the article goes in the current thread
        """         
        if self.isThreadParent(): return 0

        depth = self.getNeoBoardArticleURL().count( '/' )
        if depth > self.getNeoBoard().getMaxThreadDepth():
            return self.getNeoBoard().getMaxThreadDepth()
        else:
            return depth

    security.declarePublic( 'isThreadParent' )
    def isThreadParent( self ):
        """
            Tell if the article is top-most one in the current thread
        """                     

        return hasattr( aq_base( self.aq_inner.aq_parent ), 'neoboard' )

    security.declarePublic( 'getNeoBoardArticleNum' )
    def getNeoBoardArticleNum( self ):
        """
            Return article number
        """                     
        if self.isThreadParent():
            return self.getNeoPortalElementNumber()
        else:
            return '-'

    security.declarePublic( 'getNumAttachments' )
    def getNumAttachments( self ):
        """
            Return the number of attachments
        """     
        return len( self.getAttachments() )
    
    security.declarePublic( 'hasChildren' )
    def hasChildren( self ):
        """
            Tell if the article has replies
        """                     
        return self.getNumNeoBoardArticleReplies() > 0


    security.declarePublic( 'getSlicedTitle' )
    def getSlicedTitle( self, force=0 ):
        """
            Return title reduced to Maximum List Title Length
        """
        
        # getSlicedTitle is a very expensive call, cache its result when editing article
        
        if hasattr( aq_base( self ), '_nb_article_sliced_title' ) and not force:
            return self._nb_article_sliced_title        
         
        depth = self.getNeoBoardArticleDepth()
        
        chunk = self.getMaxListTitleLength() - ( self.getNeoBoardArticleDepth() * 2 )
        
        self._nb_article_sliced_title = cutString( self.Title(), chunk )
        
        return self._nb_article_sliced_title


    security.declarePublic( 'getNextArticleReplyNum' )
    def getNextArticleReplyNum( self ):
        """
            Return next reply article number
        """     

        return self.resetElementNumbers( e_type = NeoBoardArticle.meta_type )

    security.declarePublic( 'getNeoBoardArticleReplies' )
    def getNeoBoardArticleReplies( self ):
        """
            Return article replies
        """                     
        
        return self.searchResults( meta_type=NeoBoardArticle.meta_type, isTempNeoBoardArticle=0, getSortKey=self.getSortKey(), isThreadParent=0, sort_on='getSortKeys' )

    security.declarePublic( 'getNumNeoBoardArticleReplies' )
    def getNumNeoBoardArticleReplies( self ):
        """
            Return the total number of replies of this article
        """                     
        return len( self.getNeoBoardArticleReplies() )
        

    security.declareProtected( NeoPortalManageProperties, 'setNeoBoardArticleCategory' )
    def setNeoBoardArticleCategory( self, category ):
        """
            Set the category of this article
        """                                     

        if self.isThreadParent():
            self.category=category
        else:
            self.category = self.getNeoBoardArticleThreadParent().category
    
    security.declarePublic( 'getNeoBoardArticleParent' )
    def getNeoBoardArticleParent( self ):
        """
            Return parent of this article
        """             
        if self.isThreadParent():
            return None
            
        return self.aq_inner.aq_parent

    security.declarePrivate( 'getNeoBoardArticleThreadParent' )
    def getNeoBoardArticleThreadParent( self ):
        """
            Return top-most parent of this article
        """                     
        if self.isThreadParent():
            return self
            
        tp_id = self.getNeoBoardArticleURL().split( '/' )[0]        
        return self.getNeoBoard()._getOb(tp_id)

    security.declarePublic( 'manage_addNeoBoardArticleReply' )
    def manage_addNeoBoardArticleReply(self
            , creator=''
            , password=''
            , creator_email=''
            , title=''
            , text=''
            , attachments=''
            , mail_reply=0
            , check_filepath=1
            , REQUEST=None ):
        """ 
            Add NeoBoard Article Reply: need to be public to allow anonymous posting
        """                

        id = NeoBoardArticle.neoportal_element_prefix + str( self.getNextArticleReplyNum() )
        instance = NeoBoardArticle( id=id, title=title )                
        instance._is_temp_article=1
        
        self._setObject( id, instance )
                
        if self.getNeoBoard().emptyPasswordAllowed() and not password:
            password = self._generatePassword()

        result = self._getOb(id)._edit( creator=creator
                 , password=password
                 , creator_email=creator_email
                 , title=title
                 , text=text
                 , attachments=attachments
                 , mail_reply=mail_reply
                 , manage_edit=1
                 , check_filepath=check_filepath
                 , OBJ=self
                 , REQUEST=REQUEST )
        if result:
            self.manage_delObjects( [id,] )
            return result
        
        if self.autoApproveArticle():
             self._getOb(id).approved = 1

        del self._getOb(id)._is_temp_article
        
        self.getNeoBoard().updateNeoPortalElementContainerCatalogIndex( self._getOb(id) )
        
        if self.isMailEnabled():
            self._getOb(id).mailNeoBoardContent( REQUEST=REQUEST )

        # spam blocking
        self.validateDuplicateNeoPortalContentPosting( REQUEST=REQUEST )
            
        if self.mail_reply:
            self.mailNeoBoardContent( content_obj=self._getOb(id), REQUEST=REQUEST )

        self._getOb(id).creation_date = DateTime()
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Article Replied!' )
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self._getOb(id).absolute_url(), message, query_str ) )

    security.declarePublic( 'manage_addInlineComment' ) 
    def manage_addInlineComment( self
                                , p_index=0
                                , creator=''
                                , password=''
                                , creator_email=''
                                , text=''
                                , REQUEST=None ):
        """
            Add Inline Comment
        """
        
        comment_named_anchor = '#' + NEOBOARD_REQUEST_COMMENT_PREFIX + '%s' % str( self.getNextCommentNum() )

        if self.getNeoBoard().emptyPasswordAllowed() and not password:
            password = self._generatePassword()
        
        result = addNeoBoardArticleComment( self
                                , p_index=p_index
                                , creator=creator
                                , password=password
                                , creator_email=creator_email
                                , text=text
                                , REQUEST=REQUEST )

        if result is not None:
            return result

        self._updateNeoPortalContent()
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Article Commented!' )
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s%s' % ( self.absolute_url(), message, query_str, comment_named_anchor  ) )
            
    security.declareProtected( NeoPortalView, 'manage_deleteNeoBoardArticle' ) 
    def manage_deleteNeoBoardArticle( self
                                    , password=None
                                    , REQUEST=None ):
        
        """
            Delete NeoBoard Article
        """

        if ( not self.useACLUserAuth() and not self.isNeoBoardManager() ) \
           or ( self.useACLUserAuth() and self.isAnonymousNeoPortalUser() ):
            if not password:
                return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )
            
            if not self.checkPassword( password ):
                return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='password', message="Password Mismatch!" )

        if self.hasChildren() and not self.isNeoBoardManager():
            return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='default', message="You can't delete an article with replies!" )

        if self.hasInlineComments() and not self.isNeoBoardManager():
            return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='default', message="You can't delete an article with inline comments!" )

        if self.useACLUserAuth():
            if self.isAnonymousNeoPortalUser() and not self.checkPassword( password ):
                return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='password', message="You don't have permissions to delete this article." )
            if not self.isNeoPortalContentOwner() and not self.isNeoBoardManager():
                return self.manageNeoPortalError( obj=self, REQUEST=REQUEST, field='password', message="You don't have permissions to delete this article." )

                
        parent = self.aq_inner.aq_parent
        parent.getNeoBoard().deleteFromNeoPortalElementContainerCatalog( self )
        parent._delObject( self.getId() )
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Article deleted!' )
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( parent.absolute_url(), message, query_str ) )

    delete = manage_deleteNeoBoardArticle

    security.declareProtected( NeoPortalView, 'manage_modifyNeoBoardArticle' ) 
    def manage_modifyNeoBoardArticle( self
            , creator=''
            , password=''
            , creator_email=''
            , title=''
            , text=''
            , category=''
            , attachments=[]
            , mail_reply=0
            , check_filepath=1
            , REQUEST=None ):
        """ 
            Modify NeoBoard Article
        """

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , title=title
                     , text=text
                     , category=category
                     , attachments=attachments
                     , mail_reply=mail_reply
                     , manage_edit=0
                     , check_filepath=check_filepath
                     , REQUEST=REQUEST )

        if result:
            return result
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )
            

    security.declareProtected( NeoPortalView, 'manage_modifyNeoBoardArticleComment' )
    def manage_modifyNeoBoardArticleComment( self, 
            comment_id,
            creator='',
            password='',
            creator_email='',
            text='',
            REQUEST=None ):
            """
                Modify comment
            """
            
            comment = self.getInlineComment( id=comment_id )
            if comment is not None:
                result = comment.edit( creator=creator
                    , password=password
                    , creator_email=creator_email
                    , text=text
                    , REQUEST=REQUEST )
                
                if result is not None:
                    return result

                if REQUEST is not None:
                    message = self.getNeoPortalMessage( message='Changes applied!' )
                    query_str = self.getFullArticleQuery( REQUEST )
                
                    return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )

            return apply( self.neoboard_main, ( self, self.manageNeoPortalUserMessage( self._cookREQUEST( REQUEST, 1 ), message='No Such Comment!' ) ) )
            
            if REQUEST is not None:
                message = self.getNeoPortalMessage( message='No Such Comment!' )
                query_str = self.getFullArticleQuery( REQUEST )
                
                return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )
            
    security.declareProtected( NeoPortalView, 'manage_deleteNeoBoardArticleComment' )
    def manage_deleteNeoBoardArticleComment( self, 
            comment_id,
            password=None,
            REQUEST=None ):
            """
                Delete Modify comment
            """
            
            comment = self.getInlineComment( comment_id )
            if comment is not None:
                result = comment.manage_deleteNeoBoardArticleComment( password=password, REQUEST=REQUEST )
                
                if result is not None:
                    return result

                if REQUEST is not None:
                    message = self.getNeoPortalMessage( message='Commend deleted!' )
                    query_str = self.getFullArticleQuery( REQUEST )
                
                    return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )

            if REQUEST is not None:
                message = self.getNeoPortalMessage( message='No Such Comment!' )
                query_str = self.getFullArticleQuery( REQUEST )
                
                return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )

    security.declarePrivate( 'getNextFileID' )
    def getNextFileID( self ):
        """
            Return next attachment file ID 
        """                             
                
        next_id = self.resetElementNumbers( e_type = NeoBoardArticleAttachment.meta_type )

        if next_id == 1: return NeoBoardArticleAttachment.neoportal_element_prefix + '1'
                
        # find an empty slot
        for id in xrange( 1, next_id ):
            if not hasattr( self, NeoBoardArticleAttachment.neoportal_element_prefix + str(id) ):
                return NeoBoardArticleAttachment.neoportal_element_prefix + str(id)
        
        return NeoBoardArticleAttachment.neoportal_element_prefix + str( next_id )
    
    security.declarePublic( 'hasAttachment' )
    def hasAttachment( self ):
        """
            Tell if the article has an attachment
        """                             
        return  self.getNumAttachments() > 0

    security.declarePublic( 'getAttachments' )
    def getAttachments( self ):
        """
            Return all attachments
        """         
        return self.objectValues( [ NeoBoardArticleAttachment.meta_type, ] )

    security.declarePublic( 'getNumAttachments' )
    def getNumAttachments( self ):
        """
            Return the nubmer of all attachments
        """         
        return len( self.getAttachments() )


    security.declarePublic( 'getAttachment' )
    def getAttachment( self, id=None ):
        """
            Return attached file
        """
        
        if id is None: 
            id = NeoBoardArticleAttachment.neoportal_element_prefix + '1'
        else: 
            id = NeoBoardArticleAttachment.neoportal_element_prefix + str( id )
        
        if hasattr( aq_base( self ), id ):
            return getattr( self, id )
        
        return None
        
    security.declarePublic( 'getAttachmentTypeIcon' )
    def getAttachmentTypeIcon( self, id=None ):
        """
            Return attached file
        """                             
        attachment = self.getAttachment( id=id )
        if attachment is None:
            return '-'
        else:
            return attachment.getAttachmentTypeIcon()

    security.declarePublic( 'getAttachmentURL' )
    def getAttachmentURL( self, id=1):
        """
            Return the URL of the named attached file
        """                             

        return self.absolute_url() + '/' + NeoBoardArticleAttachment.neoportal_element_prefix + str(id)

    security.declarePublic( 'getAttachmentTitle' )
    def getAttachmentTitle( self, id=1):
        """
            Return the title of the attached file
        """                                     

        try:
            file = self.getAttachment( id=id )
            return file.Title()
        except: 
            return ''

    security.declarePublic( 'getAttachmentDescription' )
    def getAttachmentDescription( self, id=1 ):
        """
            Return the description of the attached file
        """                                     
        
        try:
            file = self.getAttachment( id=id )
            return file.Description()
        except: 
            return ''

    security.declarePublic( 'getNeoBoardArticleCategory' )
    def getNeoBoardArticleCategory( self ):
        """
            Return the category of this article
        """                                     
        try:
            return self.getNeoBoardArticleThreadParent().category
        except:
            return None

    security.declarePublic( 'getNeoBoardArticleCategoryName' )
    def getNeoBoardArticleCategoryName( self ):
        """
            Return the name of the article's category
        """                                     
        
        cat = self.getNeoBoardArticleCategory()
        
        if cat:
            try:
                return self.posting_categories[cat]
            except KeyError:
                return ''
        else:
            return ''

    security.declarePublic( 'getNeoBoardArticleSummary' )
    def getNeoBoardArticleSummary( self, size=250, ellipsis='...', force=0 ):
        """
            Return article summary
        """
        
        # getNeoBoardArticleSummary is a very expensive call, cache its result when editing article
        
        if hasattr( aq_base( self ), '_nb_article_summary' ) and not force:
            return self._nb_article_summary
        
        try:
            summary = html_quote( self.getRenderedNeoPortalContent( view_comment=0 ) ).replace( '"', ' ' )
            
            if len(summary) < size:
                ret_size = len(summary)
            else:
                ret_size = summary.find( "\n\n", 0, size )
                if ret_size == -1:
                    ret_size = summary.rfind( " ", 0, size )
                    if ret_size == -1:
                        ret_size = size

            self._nb_article_summary = removeTags( summary[:ret_size] + ellipsis )
            return self._nb_article_summary
        except:
            return self.Title()
    
    security.declarePublic( 'isApproved' )
    def isApproved( self ):
        """
            Tell if the article is approved
        """     
        if hasattr( self, 'approved' ):
            return self.approved
        else:
            return 0

    security.declarePublic( 'getAttachmentsSearchText' )
    def getAttachmentsSearchText( self ):
        """
            Catalog TextIndex
        """
        
        lines = [ attachment.getFileName() + ' ' + attachment.Title() + ' ' + attachment.Description() for attachment in self.getAttachments() ]
        
        return removeTags( ' '.join(lines))            
            
    ###################################################################################
    # 
    #       Manage Methods 
    #
    ###################################################################################            

    security.declareProtected( NeoPortalManageProperties, 'manage_editNeoBoardArticle' )
    def manage_editNeoBoardArticle( self
            , creator=''
            , password='' 
            , creator_email=''
            , title=''
            , text=''
            , category=''
            , mail_reply=0
            , check_filepath=1
            , attachments=[]
            , approved=0
            , REQUEST=None ):
        """ 
            Edit NeoBoard Article
        """

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , title=title
                     , text=text
                     , category=category
                     , attachments=attachments
                     , mail_reply=mail_reply
                     , manage_edit=1
                     , check_filepath=check_filepath
                     , approved=approved    
                     , REQUEST=REQUEST )

        if result:
            return result
        
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST )
            
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )

    security.declareProtected( NeoPortalView, 'edit' )
    def edit( self
             , creator=''
             , password=''
             , creator_email=''
             , title=''
             , text=''
             , category=''
             , attachments=[]
             , mail_reply=0
             , check_filepath=1
             , REQUEST=None ):

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , title=title
                     , text=text
                     , category=category
                     , attachments=attachments
                     , OBJ=self
                     , mail_reply=mail_reply
                     , check_filepath=check_filepath
                     , REQUEST=REQUEST )

        if result:
            return result

        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST )
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )
                     
    security.declarePrivate( '_edit' )
    def _edit( self
             , creator=''
             , password=''
             , creator_email=''
             , title=''
             , text=''
             , category=''
             , attachments=[]
             , mail_reply=0
             , OBJ=None
             , manage_edit=0
             , approved=0     
             , check_filepath=1
             , REQUEST=None ):       
                
        """                
            in _edit() method _sort_key attribute is created.
        
            threaded look requires two sort keys: 
            
                - thread parents are first sorted on this _sort_key attr (negative number for automatic reverse sorting)
                
                - thread parents' kids are then sorted on '_thread_sort_key' , which is basically absolute_url - neoboard_url
        """
        
        if not hasattr( aq_base( self ), '_sort_key' ) and self.getId().startswith( NeoBoardArticle.neoportal_element_prefix ):
            self._setSortKey()

        if not hasattr( aq_base( self ), '_thread_sort_key' ) and self.getId().startswith( NeoBoardArticle.neoportal_element_prefix ):
            self._setThreadSortKey()

        if REQUEST is None:
            REQUEST = self.REQUEST
            
        if self.getNeoBoard().isReadOnly() and not self.isNeoBoardManager():
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="This is a read-only board!" )
            
        # ACL User authentication is in use
        if self.useACLUserAuth():
            # not a manager nor an owner - don't mess with others' articles
 
            if self.isAnonymousNeoPortalUser():
                
                if not manage_edit and not password:
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )
                
                if not manage_edit and not self.checkPassword( password ):
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="Password Mismatch!" )

                # anonymous posting: set password
                if manage_edit and not self.checkPassword( password ):
                    self.setPassword( password )

            if not manage_edit:
                if self.isAnonymousNeoPortalUser() and not self.checkPassword( password ):
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="You don't have permissions to modify this article." )
                if not self.isNeoPortalContentOwner() and not self.isNeoBoardManager():
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="You don't have permissions to modify this article." )
                    
            if manage_edit and creator:
                acl = getattr(self.getNeoBoard(), 'acl_users', None)
                if acl is None:
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="User Folder not found!" )          
                
                if acl.getUser(creator.strip()) is None and not self.anonymousPostingAllowed():
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="No such user!" )          

                self.setCreator(creator.strip())
            else:
                self.setCreator(self.getAuthenticatedNeoPortalUser().getUserName())
                
        else:
            if not creator:
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator', message="The field cannot be empty!" )            

            self.setCreator( creator )
            
            # manager? allow him/her to change the article's password
            if manage_edit and not self.checkPassword( password ):
                self.setPassword( password )
                        
            if not password and not self.isNeoBoardManager():
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )            

            if not manage_edit and not self.checkPassword( password ) and not self.isNeoBoardManager():           
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="Password Mismatch!" )                            
                
        # empty email address not allowed
        if not self.emptyEmailAllowed():
            if not creator_email:
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="The field cannot be empty!" )            
        
            if not isEmailValid(creator_email.strip()):
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="Invalid Email Address!" )
        else:
            if creator_email.strip() and not isEmailValid(creator_email.strip()):
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="Invalid Email Address!" )

                
        # not a manager nor an owner - article has yet to be approved, not viewable
        if not manage_edit and not self.isNeoBoardManager() and not self.isNeoPortalContentOwner() and not self.isApproved():
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="The article has yet to be approved!" )          

        # empty title not allowed
        if not title:
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="The field cannot be empty!" )               
            
        # empty body allowed? if not, yell at him/her
        if not self.emptyBodyAllowed() and not text:
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='text', message="The field cannot be empty!" )                
                
        self.setTitle(title)
        self.setRawNeoPortalContent(text)
        self.mail_reply=mail_reply
               
        self.setCreatorEmail(creator_email)
        
        # only thread parents can have a category
        self.setNeoBoardArticleCategory(category)
        
        # got attachments?
        if self.attachmentsAllowed():
            # rigirously validate attachments: some browers mess with file upload instances

            attached = 0
            for attachment in attachments:
                saved_attachment = None
                if attachment and attachment.has_key( 'index' ) and attachment['index']:
                    saved_attachment = self.getAttachment(attachment['index'])                    

                if saved_attachment is not None:
                    if attachment and attachment.has_key( 'file' ) and not attachment['file'] \
                       and attachment.has_key( 'title' ) and not attachment['title'] \
                       and attachment.has_key( 'description' ) and not attachment['description']:
                       # existing slot but title/description got emptied, delete the attachment
                       
                        try:
                            self.manage_delObjects([saved_attachment.getId(),])
                        except:
                            self.np_log( message="Failed to delete attachment %s from %s" % ( saved_attachment.getId(), self.absolute_url() ), summary='NeoBoardArticle' )            
                    else:
                        # modify title/description with the given values
                        if attachment.has_key( 'title' ) and attachment['title']:
                            saved_attachment.title = attachment['title']
                            
                        if attachment.has_key( 'description' ) and attachment['description']:
                            saved_attachment.description = attachment['description']
                
                if attachment and attachment.has_key( 'file' ) and attachment['file']:
                    attached = 1
            
            if attached:
                
                # check filesize total
                f_list = []
                extra=0
                
                # already have attachments? get them into equation
                for attachment in self.getAttachments():
                    extra = extra + attachment.getFileSize()

                # check new ones
                
                idx = 0
                for attachment in attachments:
                    idx = idx + 1
                    
                    # filter out empty attachment slot
                    # fileupload instance is supposed to evaluate to false if 'filename' is empty
                    
                    if not attachment.has_key( 'file' ) or \
                       not attachment.get( 'file', None) or \
                       not attachment['file']: continue
                    
                    # take existing slots into account
                    t_file = self.getAttachment(idx)
                    if t_file is not None:
                        extra = extra - t_file.getFileSize()

                    if attachment['file'].filename:
                        # that's a baaa~~d extension~
                        if os.path.splitext(attachment['file'].filename)[1] in self.getBadExtensions():
                            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='attachment', message="The following extensions are not allowed: %s.", tup= ' '.join(self.getBadExtensions() ) )
                    
                        f_list.append(attachment['file'])
        
                # yo! you gots some big files!
                
                if self._attachmentsTooBig(f_list, extra=extra):
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='attachment', message="The total size of the uploaded file(s) exceed the limit." )
                
                idx = 0
                for attachment in attachments:
                    idx = idx + 1
                    
                    if not attachment.has_key( 'file' ) or \
                       not attachment.get( 'file', None) or \
                       not attachment['file']: continue
                    
                    if hasattr(aq_base(self), NeoBoardArticleAttachment.neoportal_element_prefix + str(idx)):
                        fid = NeoBoardArticleAttachment.neoportal_element_prefix + str(idx)
                    else:
                        fid = self.getNextFileID()
                        
                    if attachment['file'].filename:
                        title = ''
                        description = '' 
                        if attachment.has_key( 'title' ):
                            title = attachment['title']
                        if attachment.has_key( 'description' ):
                            description = attachment['description']
                            
                        addNeoBoardArticleAttachment( self, id=fid, title=title, description=description, file=attachment['file'], check_filepath=check_filepath )
        
        if manage_edit:
            self.approved=approved
        
        # flush cache and reindex
        self._updateNeoPortalContent()
        
        # cache summary
        self.getNeoBoardArticleSummary( force=1 ) 

        # cache sliced title
        self.getSlicedTitle( force=1 ) 
        
        # manager - we can track you down baby, if we want to;-)
        self._setRemoteIP( REQUEST )
        return None

    security.declareProtected( NeoPortalManageProperties, 'manage_articleEditForm' )
    def manage_articleEditForm( self, REQUEST=None ):
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        REQUEST.set( 'neoboard_form', 'neoboard_article_edit_form' )
        return self.neoboard_main
        return apply( self.neoboard_main, ( self, self._cookREQUEST( REQUEST ) ) )

    security.declareProtected( NeoPortalManageProperties, 'manage_approve' )
    def manage_approve( self, REQUEST=None ):
        """
            Disapprove the article
        """     
        self.approved = 1
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Article Approved!' )
            query_str = self.getFullArticleQuery( REQUEST )
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )            

    security.declareProtected( NeoPortalManageProperties, 'manage_disapprove' )
    def manage_disapprove( self, REQUEST=None ):
        """
            Approve the article
        """
        
        self.approved = 0
        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Article disapproved!' )
            query_str = self.getFullArticleQuery( REQUEST )
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self.absolute_url(), message, query_str ) )   
        
    security.declareProtected( NeoPortalManageProperties, 'approveThread' )
    def approveThread( self ):
        """
            Approve all the articles down this thread
        """             
        self.approved = 1
        
        dummy = [ result.getObject().manage_approve() for result in self.getNeoBoardArticleReplies() ]

    security.declareProtected( NeoPortalManageProperties, 'disApproveThread' )
    def disApproveThread( self ):
        """
            Disapproves all the articles down this thread
        """             
        self.approved = 0
        
        dummy = [ result.getObject().manage_disapprove() for result in self.getNeoBoardArticleReplies() ]

    security.declarePrivate( '_attachmentsTooBig' )
    def _attachmentsTooBig( self, files, extra=0 ):
        """
            Tell if the given set of attachments exceeds the size limit
        """             
        t_size = extra
        for file in files:
            file.seek(0)
            t_size = t_size + len( file.read() )
            file.seek(0)
        
        # in MB

        return int( t_size ) > ( int(self.getMaxFileSize() ) * 1047576 )

    security.declarePublic( 'getSortKeys' )
    def getSortKeys( self ):
        """
            Return Sort Key/Thread Sort Key as tuple
        """             

        return ( self.getSortKey(), self.getThreadSortKey() )
        
    security.declarePublic( 'getSortKey' )
    def getSortKey( self ):
        """
            Return Sort Key
        """             

        return self._sort_key

    security.declarePublic( 'getThreadSortKey' )
    def getThreadSortKey( self ):
        """
            Return Thread Sort Key
        """             

        return self._thread_sort_key

    security.declarePrivate( '_setSortKey' )
    def _setSortKey( self ):
        """
            Set Sort Key
        """             

        self._sort_key = -int( self.getNeoBoardArticleThreadParent().getId().replace( NeoBoardArticle.neoportal_element_prefix, '' ) )

    security.declarePrivate( '_setThreadSortKey' )
    def _setThreadSortKey( self ):
        """
            Set Thread Sort Key
        """             

        self._thread_sort_key = self.absolute_url().replace( self.getNeoBoard().absolute_url(), '' )
        
    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd( self, item, container ):
        """
            Make sure it gets cataloged
        """             

        if item is self and item.isThreadParent():        
            item._setSortKey()
            item._setThreadSortKey()
            neoboard = item.getNeoBoard()
            neoboard.updateNeoPortalElementContainerCatalogIndex( item )
            
            articles = [ obj for id, obj in item.ZopeFind( item, obj_metatypes = [ NeoBoardArticle.meta_type, ], search_sub=1 ) ]                    
            for article in articles:
                article._setSortKey()
                article._setThreadSortKey()
                neoboard.updateNeoPortalElementContainerCatalogIndex( article )        


    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete( self, item, container ):
        """
            Make sure it gets uncataloged
        """             
                
        if item is self:
            neoboard = item.getNeoBoard()
            neoboard.deleteFromNeoPortalElementContainerCatalog( item )
            if item.isThreadParent():
                dummy = [ neoboard.deleteFromNeoPortalElementContainerCatalog( obj ) for id, obj in item.ZopeFind( item, obj_metatypes = [ NeoBoardArticle.meta_type, ], search_sub=1 ) ]                   
                            
InitializeClass( NeoBoardArticle )

class NeoBoardNoticeArticle( NeoBoardArticle ):
    """
        NeoBoard Notice Article
    """
    
    neoportal_element_prefix = 'n_' # should end with '_' and should not contain any more '_'    
    neoportal_cacheable_content = 1
    neoportal_inline_commentable_content = 0
    neoportal_spam_blockable_content = 0
    neoportal_iconizable_content = 1
    neoportal_ratable_content = 0
    neoportal_discussable_content = 0
    
    meta_type='NeoBoard Notice Article' 
    
    security = ClassSecurityInfo()
    
    security.declarePublic( 'getNeoBoardArticleReplies' )
    def getNeoBoardArticleReplies( self ):
        """
            Return None
        """                     
        from Products.ZCatalog.Lazy import LazyCat
        return LazyCat( [] )
    
    
    security.declarePrivate( '_setSortKey' )
    def _setSortKey( self ):
        """
            Set Sort Key
        """             

        self._sort_key = -int( self.getNeoBoardArticleThreadParent().getId().replace( NeoBoardNoticeArticle.neoportal_element_prefix, '' ) )    

    security.declarePublic( 'isNeoBoardArticle' )
    def isNeoBoardArticle( self ):
        """
            Tell if self is an article
        """
        return self.meta_type == NeoBoardNoticeArticle.meta_type

    security.declarePrivate( 'setPassword' )
    def setPassword( self, password ):
        """
            Just give it a random password since managers don't need one
        """ 
        
        self.password = encryptNeoPortalPassword( self._generatePassword() )
        
    security.declarePublic( 'inlineCommentsAllowed' )
    def inlineCommentsAllowed( self ):
        """
           Never allow inline commenting in notices
        """
        
        return 0

    security.declarePublic( 'contentRatingAllowed' )        
    def contentRatingAllowed( self ):
        """
           Never allow content rating for notices
        """

        return 0
    
InitializeClass( NeoBoardNoticeArticle )

def addNeoBoardArticle( self
        , creator=''
        , password=''
        , creator_email=''
        , title=''
        , text=''
        , category=''
        , attachments=[]
        , mail_reply=0
        , is_notice=0
        , check_filepath=1        
        , REQUEST=None ):
    """ 
        Add NeoBoard Article
    """

    if is_notice:
        id = NeoBoardNoticeArticle.neoportal_element_prefix + str( self.getNextArticleNum( notice=1 ) )
        instance = NeoBoardNoticeArticle( id=id, title=title, text=text )

    else:
        id = NeoBoardArticle.neoportal_element_prefix + str( self.getNextArticleNum() )
        instance = NeoBoardArticle( id=id, title=title, text=text )

    instance._is_temp_article=1
    instance._setRemoteIP( REQUEST )

    self.getNeoBoard()._setObject( id, instance )

    if self.getNeoBoard().emptyPasswordAllowed() and not password:
        password = self.getNeoBoard()._getOb(id)._generatePassword()

    result = self.getNeoBoard()._getOb(id)._edit( creator=creator
             , password=password
             , creator_email=creator_email
             , title=title
             , text=text
             , category=category
             , attachments=attachments
             , manage_edit=1
             , mail_reply=mail_reply
             , check_filepath=check_filepath
             , OBJ=self.getNeoBoard()
             , REQUEST=REQUEST )             

    if self.autoApproveArticle():
        self.getNeoBoard()._getOb(id).approved=1
    else:
        self.getNeoBoard()._getOb(id).approved=0             

    if result:
        self.getNeoBoard().manage_delObjects([id,])
        return result        

    del self.getNeoBoard()._getOb(id)._is_temp_article
    self.getNeoBoard().updateNeoPortalElementContainerCatalogIndex( self.getNeoBoard()._getOb(id) )
    
    # spam blocking
    self.validateDuplicateNeoPortalContentPosting( REQUEST=REQUEST )

    if self.isMailEnabled():
        self._getOb(id).mailNeoBoardContent( REQUEST=REQUEST )        

    self._getOb(id).creation_date = DateTime()
    
    if REQUEST is not None:
        message = self.getNeoPortalMessage( message='Article added!' )
        query_str = self.getFullArticleQuery( REQUEST )
    
        return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( self._getOb(id).absolute_url(), message, query_str ) )           