#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardArticleAttachment.py,v 1.15 2003/04/01 17:48:15 wankyu Exp $
Description

    NeoBoard Article Attachment Wrapper: OFS.Image.File or NeoPortal ExternalFile
"""

import os, sys
from Globals import package_home, InitializeClass

from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent, aq_base, aq_inner
from OFS.Image import File, Image, getImageInfo

from Products.NeoPortalLibrary.NP_BaseExternalFile import NeoPortalBaseExternalFile, addNeoPortalBaseExternalFile, copyFile, checkFilePath
from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_Element import NeoPortalElement
from Products.NeoPortalLibrary.NP_IconizableContent import NeoPortalIconizableContent
from Products.NeoPortalLibrary.NP_EmbeddableContent import NeoPortalEmbeddableContent
from Products.NeoPortalLibrary.NP_TranslatableContent import NeoPortalTranslatableContent
from Products.NeoPortalLibrary.NP_UpgradableObject import NeoPortalUpgradableObject

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import os, mimetypes, string, types

from Products.NeoBoard.NeoBoard import __NEOPORTAL_CLASS_VERSION__

#
# adapted from ExtFile: ExtFile's TRANSMAP messes up with Asian characters!
#
bad_chars =  ' ,;()[]{} `~!@#$%^&*+='
good_chars = '______________________'
TRANSMAP = string.maketrans( bad_chars, good_chars )

def addNeoBoardArticleAttachment( self
        , id
        , title=''
        , description=''
        , file=''
        , check_filepath=1 ):
    """ 
        Add NeoBoard Article Attachment
    """

    if type( file ) is types.StringType: 
        desc_filename = os.path.basename( file.replace('\\', '/') )
    else:
        desc_filename = os.path.basename( file.filename.replace('\\', '/') )
        
    archive_filename = string.translate( desc_filename, TRANSMAP )

    if not title:
        title = archive_filename
    if not description:
        description = archive_filename
    
    # the slot already occuppied, replace it    
    if hasattr( aq_base( self ), id ):
        slot = self._getOb(id)

        if file:
            # title/desc were set to desc_filename, change them
            if slot.Title() == slot.getFileName():
                slot.setTitle( archive_filename )

            if slot.Description() == slot.getFileName():
                slot.setDescription( archive_filename )
        
            slot.manage_upload( file=file, archive_filename=archive_filename )
        else:
            slot.setTitle( title )
            slot.setDescription( description )
            
        return            
       
    # External File
    if self.useFileSystem():
        
        archive_filepath = self.getArchivePath() + self.getNeoBoardArticleURL()

        if check_filepath and not checkFilePath( archive_filepath ):
            raise 'NeoBoardError', "Unauthorized filepath base!"
        
        if not os.path.exists( archive_filepath ):
            try:
                os.makedirs( archive_filepath )
            except IOError:
                raise 'NeoBoardError', "Can't create the attachment file dir: %s" % archive_filepath

        archive_filepath = archive_filepath + os.sep +  id + '_' + archive_filename
        
        instance = NeoBoardArticleAttachmentInExternalFile( filepath=archive_filepath, id=id, title=title, description=description, file=file, check_filepath=check_filepath )
        #
        # NeoPortal Element Container automatically resets file ordering unless no_id_check is set to true
        #
        self._setObject( id, instance, no_id_check=1 )

    else:
        instance = NeoBoardArticleAttachmentInFile( id=id, title=title, description=description, file=file, archive_filename=archive_filename)
        #
        # NeoPortal Element Container automatically resets file ordering unless no_id_check is set to true
        #
        
        self._setObject( id, instance, no_id_check=1 )

    self.getNeoBoard().updateNeoPortalElementContainerCatalogIndex( self._getOb( id ) )            

class NeoBoardArticleAttachment( NeoPortalBaseItem
                               , NeoPortalElement
                               , NeoPortalIconizableContent
                               , NeoPortalEmbeddableContent
                               , NeoPortalTranslatableContent
                               , NeoPortalUpgradableObject ):
    """
        NeoBoard Article Attachment Wrapper
    """

    __NEOPORTAL_CLASS_VERSION__  = __NEOPORTAL_CLASS_VERSION__

    neoportal_element_prefix = 'attachment_' # should end with '_' and should not contain any more '_'

    _np_element_renumbering_allowed = 0
    
    _np_content_can_be_embedded = 1
    _np_content_can_embed = 0    
    
    meta_type='NeoBoard Article Attachment Element'

    security = ClassSecurityInfo()

    security.declarePublic( 'attachmentCallView' )
    def attachmentCallView( self, REQUEST=None ):
        """
            Return attachment's call view to use for embedding
        """
        
        if REQUEST is None:
            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
            else:
                return ''
              
        view = self.neoboard_article_attachment_call_view

        if getattr( aq_base( view ), 'isDocTemp', 0 ):
            return apply( view, ( self, REQUEST ) )
        else:
            return view()        
        
    call_view = attachmentCallView
    
    security.declarePublic( 'isNeoBoardArticleAttachment' )
    def isNeoBoardArticleAttachment( self ):
        """
            Tell if self is an article attachment
        """
        return self.meta_type == NeoBoardArticleAttachment.meta_type
    
    security.declarePublic( 'getAttachmentTypeIcon' )
    def getAttachmentTypeIcon( self ):
        """
            Return attachment mime type icon
        """ 

        return self.getMimeTypeIcon( mime_type = self.getContentType() )
        
    security.declarePublic( 'isNeoPortalExternalFile' )
    def isNeoPortalExternalFile( self ):
        """
            Tell if self is a NeoPortal ExternalFile instance
        """
        return hasattr( aq_base( self ), 'neoportal_external_file' ) and self.neoportal_external_file
        
class NeoBoardArticleAttachmentInExternalFile( NeoPortalBaseExternalFile, NeoBoardArticleAttachment ):
    """
        NeoBoard Article Attachment in ExternalFile
    """    
        
    meta_type = NeoBoardArticleAttachment.meta_type
    
    security = ClassSecurityInfo()
    
    def __init__ (self
                , filepath
                , id=''
                , title=''
                , description=''
                , file=''
                , check_filepath=1 ):
        
        self._mime_icons = {}

        NeoPortalBaseExternalFile.__init__( self
                        , filepath=filepath
                        , id=id
                        , title=title
                        , description=description
                        , file=file
                        , check_filepath=check_filepath )
                        
        self._setCurrentClassVersion( __NEOPORTAL_CLASS_VERSION__ )                        
                        
    security.declarePublic( 'getId' )
    def getId( self ):
        """
            Just check the id in case the instance is in a temporary state, in which case no id gets set
        """
        id = NeoPortalBaseExternalFile.getId( self ) 
        if not id:
            id = self.id = self.__name__

        return id                        

    security.declarePrivate( 'manage_afterClone' )
    def manage_afterClone(self, item, new_fn=None):
        """ 
            Copy and pasted: NeoPortal ExternalFile is a 'hard link'. A copy of the file should also be created.
        """
        
        self._checkFile()

        return NeoPortalBaseExternalFile.inheritedAttribute ("manage_afterClone") (self, item)
                
    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd(self, item, container):
        """
            Tell if self is being copied
        """             

        self._checkFile()
        item.getNeoPortalElementContainer().updateNeoPortalElementContainerCatalogIndex( item )
        return NeoPortalBaseExternalFile.inheritedAttribute ("manage_afterAdd")(self, item, container)  
        
    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete( self, item, container):
        """
            Make sure it gets uncataloged
        """             
        
        item.getNeoPortalElementContainer().deleteFromNeoPortalElementContainerCatalog( item )        
        return NeoPortalBaseExternalFile.inheritedAttribute ("manage_beforeDelete")(self, item, container)  

    security.declarePrivate( '_checkFile' )
    def _checkFile( self ):
        """
            Copy or move filepath: when .temp file exists in self.filepath, that means self is being 'cut' and pasted
        """        

        archive_filepath = self.getArchivePath() + self.getNeoBoardArticleURL()
            
        if not os.path.isfile(archive_filepath + os.sep +  self.getFileName()):   # I'm being copied!
            try:
                os.makedirs(archive_filepath)
            except:
                pass
                
            temp_filename = self._getTempFileName()
            
            if not os.path.isfile(self.getFilePath()) and os.path.isfile(temp_filename):
                
                # being CUT and PASTED
                try:
                    os.rename(temp_filename, self.getFilePath())
                    remove_old_archive = 1
                    
                except:
                    raise
                    
            else:
                # being COPIED 
                
                remove_old_archive = 0
    
            new_archive_filepath = archive_filepath + os.sep +  self.getFileName()
            copyFile(self.getFilePath(), new_archive_filepath)
            
            if remove_old_archive:
            
                try:
                    os.unlink(self.getFilePath())
                except:
                    raise
                    
                old_archive=os.path.dirname(self.getFilePath())
    
                try:                                    
                    # this should raise an exception if the old_archive is not empty yet
                    # when all archived files are moved to a new location, this call would succceed
                    os.removedirs(old_archive)
                except:
                    pass
                    
            self._resetFilePath()

    security.declarePrivate( '_resetFilePath' )
    def _resetFilePath( self ):
        """
            Resets File Path in case object gets copied or moved
        """
        
        self.filepath = self.getArchivePath() + self.getNeoBoardArticleURL() + os.sep +  self.getFileName()

InitializeClass( NeoBoardArticleAttachmentInExternalFile )

class NeoBoardArticleAttachmentInFile( Image, NeoBoardArticleAttachment ):
    """
        NeoBoard Article Attachment in File (Internally use Image Class to easily support Image features)
    """
    
    meta_type = NeoBoardArticleAttachment.meta_type
    manage_options=File.manage_options 

    security = ClassSecurityInfo()
    def __init__( self
                , id=''
                , title=''
                , description=''
                , file=''
                , archive_filename='' ):
                    
        self._mime_icons = {}

        Image.__init__( self
                    ,id=id
                    ,file=file
                    ,title=title )
                        
        self.setDescription( description )
        self.archive_filename=archive_filename
        self._setCurrentClassVersion( __NEOPORTAL_CLASS_VERSION__ )                        

    security.declarePublic( 'isImage' )
    def isImage( self ):
        """
            Tell if file content is image
        """
        
        if hasattr( self, '_is_image' ):
            return self._is_image
        
        content_type, width, height = getImageInfo(self.data)

        try:
            is_image = 0
            if content_type.split('/')[0].lower() == 'image':
                is_image = 1
            self._is_image = is_image
            return is_image
        except:
            return 0

    def __str__( self ):
        if self.isImage():
            return self.tag()
        else:
            return File.__str__( self )

    security.declarePublic( 'thumb' )
    def thumb(self, REQUEST=None, RESPONSE=None):
        """
            Return a thumbnail image
        """

        return self.index_html(REQUEST=REQUEST, RESPONSE=RESPONSE)            
    
    security.declarePublic( 'index_html' )
    def index_html(self, REQUEST=None, RESPONSE=None):
        """
            Is me an image or file?
        """

        if REQUEST is None and hasattr(self,'REQUEST'): 
            REQUEST = self.REQUEST
            
        RESPONSE.setHeader( 'Content-Disposition', 'attachment; filename=%s' % self.getFileName() )       
        return Image.index_html(self, REQUEST, RESPONSE)

    security.declareProtected( NeoPortalManageProperties, 'update_data' )
    def update_data(self, data, content_type=None, size=None):
        if size is None: size=len(data)
        if content_type:
            self.content_type = content_type
            
        self.size=size
        self.data=data

        ct, width, height = getImageInfo(data)

        if width >= 0 and height >= 0:
            self.width = width
            self.height = height

        self.ZCacheable_invalidate()
    
    security.declarePublic( 'getImageSize' )
    def getImageSize( self ):
        """
            Return ImageSize
        """        
        return self.width, self.height

    security.declarePublic( 'getFileName' )      
    def getFileName( self ):
        """
            Return Filename
        """
        return self.archive_filename

    security.declareProtected( NeoPortalView,  'getFileSize' )
    def getFileSize( self ):
        """
            Returns File Size
        """     
        return self.get_size()

    security.declarePublic( 'getFileSizeInKB' )   
    def getFileSizeInKB( self ):
        """
            Returns File Size in KB
        """     
        return round( int( self.getFileSize() / 1024 ) )

    security.declareProtected( NeoPortalAddPortalContent , 'manage_upload' )
    def manage_upload( self,file='',archive_filename='', REQUEST=None ):
        """
            Change filename if not the same
        """
        self.archive_filename=archive_filename
        return Image.manage_upload( self, file=file, REQUEST=REQUEST )

    security.declarePrivate( 'manage_afterAdd' )
    def manage_afterAdd(self, item, container):
        """
            Tell if self is being copied
        """             

        item.getNeoPortalElementContainer().updateNeoPortalElementContainerCatalogIndex( item )
        return NeoBoardArticleAttachmentInFile.inheritedAttribute ("manage_afterAdd")(self, item, container)  
        
    security.declarePrivate( 'manage_beforeDelete' )
    def manage_beforeDelete( self, item, container):
        """
            Make sure it gets uncataloged
        """             
        
        item.getNeoPortalElementContainer().deleteFromNeoPortalElementContainerCatalog( item )        
        return NeoBoardArticleAttachmentInFile.inheritedAttribute ("manage_beforeDelete")(self, item, container)  
           
InitializeClass( NeoBoardArticleAttachmentInFile )