#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardArticleComment.py,v 1.9 2003/04/02 07:58:36 wankyu Exp $
Description

    NeoBoard Article Inline Comment
"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from OFS.DTMLMethod import DTMLMethod
from DateTime.DateTime import DateTime
from DocumentTemplate.DT_Util import html_quote

from Products.NeoPortalLibrary.NP_InlineComment import NeoPortalInlineComment
from Products.NeoPortalLibrary.NP_Utils import cutString, isEmailValid
from Products.NeoPortalLibrary.NP_InlineCommentableContent import  NP_INLINE_COMMENT_OBJECT_PREFIX

from Products.NeoBoard.NeoBoardBase import NeoBoardBaseItem

from Products.NeoBoard.NeoBoard import __NEOPORTAL_CLASS_VERSION__

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

import os, mimetypes

NEOBOARD_REQUEST_COMMENT_PREFIX = NP_INLINE_COMMENT_OBJECT_PREFIX + '_'

def addNeoBoardArticleComment( self, 
        p_index=0,
        creator='',
        password='',
        creator_email='',
        text='',
        REQUEST=None ):
    """ 
        Add NeoBoard Article Inline Comment
    """

    id = self._getInlineCommentPrefix() + '_' + str( self.getNextCommentNum() )
    instance = NeoBoardArticleComment( id=id, text=text )

    instance._setRemoteIP(REQUEST)
    self._addInlineComment(instance)    
            
    result = self.getInlineComment(id)._edit( creator=creator
             , password=password
             , creator_email=creator_email
             , text=text
             , manage_edit=1
             , OBJ=self
             , REQUEST=REQUEST )

    if result:
        self.deleteInlineComment( id, check_permission=0 )
        return result            

    if not self._insertInlineCommentMarker( id=id, p_index=p_index ):
        return self.manageNeoPortalError( REQUEST=REQUEST, field='default', message="The paragraph index '%s' is invalid!", tup=(p_index,) )    

    # spam blocking
    self.validateDuplicateNeoPortalContentPosting( REQUEST=REQUEST )

    self.getInlineComment(id)._setRemoteIP( REQUEST=REQUEST )
    self._updateNeoPortalContent()
        
    if self.isMailEnabled():
        self.mailNeoBoardContent( content_obj=self, is_comment=1, REQUEST=REQUEST  )

    if self.mail_reply:
        self.mailNeoBoardContent( is_comment=1, REQUEST=REQUEST )        

    return None
    
class NeoBoardArticleComment( NeoPortalInlineComment, NeoBoardBaseItem ):
    """
        NeoBoard Article Inline Comment
    """
    
    __NEOPORTAL_CLASS_VERSION__  = __NEOPORTAL_CLASS_VERSION__
    
    np_inline_comments_allowed = 0
    
    icon='misc_/NeoBoard/neoboard_article_comment_icon'
        
    security = ClassSecurityInfo()
    
    def __init__(self, id, text=''):
        self.id = id
        self.text=text
        self.password=''
        self.creation_date=DateTime()

    security.declarePrivate( 'getInlineCommentView' )        
    def getInlineCommentView( self ):
        """
            Get inline comment view
        """        
        return self.neoboard_article_comment_view            

    security.declarePrivate( 'getLostInlineCommentView' )        
    def getLostInlineCommentView( self ):
        """
            Get the view for comments that lost context (deleted, moved, etc)
        """        
        return self.neoboard_lost_article_comment_view
    
    security.declarePublic( 'isNeoBoardArticleComment' )
    def isNeoBoardArticleComment(self):
        """
            Tell if self is an article comment
        """
        return self.meta_type == NeoBoardArticleComment.meta_type
        
        
    security.declarePublic( 'manage_deleteNeoBoardArticleComment' )
    def manage_deleteNeoBoardArticleComment( self
                                , password=''
                                , REQUEST=None ):
        
        """ 
            Delete NeoBoard Article Comment
        """                                  

        if ( not self.useACLUserAuth() and not self.isNeoBoardManager() ) \
           or ( self.useACLUserAuth() and self.isAnonymousNeoPortalUser() ):
            if not password:
                return self.manageNeoPortalError( obj=self.aq_inner.aq_parent, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )

            if not self.checkPassword( password ):
                return self.manageNeoPortalError( obj=self.aq_inner.aq_parent, REQUEST=REQUEST, field='password', message="Password Mismatch!" )

        # NP_InlineComment method
        parent = self.aq_inner.aq_parent

        if self.useACLUserAuth():
            if self.isAnonymousNeoPortalUser() and not self.checkPassword( password ):
                return self.manageNeoPortalError( obj=self.aq_inner.aq_parent, REQUEST=REQUEST, field='password', message="You don't have permissions to delete this article." )
            if not self.isNeoPortalContentOwner() and not self.isNeoBoardManager():
                return self.manageNeoPortalError( obj=self.aq_inner.aq_parent, REQUEST=REQUEST, field='password', message="You don't have permissions to delete this article." )

        parent.deleteInlineComment( self.getId(), check_permission=0 )

        if REQUEST is not None:
            message = self.getNeoPortalMessage( message='Comment deleted!' )
            query_str = self.getFullArticleQuery( REQUEST )
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( parent.absolute_url(), message, query_str ) )        

    delete = manage_deleteNeoBoardArticleComment

    security.declareProtected( NeoPortalView, 'manage_modifyNeoBoardArticleComment' )
    def manage_modifyNeoBoardArticleComment( self, 
            creator='',
            password='',
            creator_email='',
            text='',
            REQUEST=None ):
        """ 
            Modify NeoBoard Article Comment
        """

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , text=text
                     , OBJ=self.aq_inner.aq_parent
                     , manage_edit=0
                     , REQUEST=REQUEST )

        if result:
            return result

        if REQUEST is not None:
            parent = self.aq_inner.aq_parent        
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST ) + '#' + self.getId()
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( parent.absolute_url(), message, query_str ) )                    

    security.declareProtected( NeoPortalView, 'edit')
    def edit( self
             , creator=''
             , password=''
             , creator_email=''
             , text=''
             , REQUEST=None ):

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , text=text
                     , OBJ=self.aq_inner.aq_parent
                     , REQUEST=REQUEST )

        if result:
            return result

        if REQUEST is not None:
            parent = self.aq_inner.aq_parent        
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST ) + '#' + self.getId()
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( parent.absolute_url(), message, query_str ) )                    
    
    security.declarePrivate( '_edit' )
    def _edit(self
             , creator=''
             , password=''
             , creator_email=''
             , text=''
             , manage_edit=0
             , OBJ=None
             , REQUEST=None):       
                
        """                
            Edit Comment
        """

        if REQUEST is None:
            REQUEST = self.REQUEST
            
        if self.getNeoBoard().isReadOnly() and not self.isNeoBoardManager():
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='title', message="This is a read-only board!" )            

        # ACL User authentication is in use
        if self.useACLUserAuth():
            
            # not a manager nor an owner - don't mess with others' comments

            if self.isAnonymousNeoPortalUser():
                if not manage_edit and not password:
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )
                
                if not manage_edit and not self.checkPassword( password ):
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="Password Mismatch!" )

                # anonymous posting: set password
                if manage_edit and not self.checkPassword( password ):
                    self.setPassword( password )
            
            if not manage_edit:
                if self.isAnonymousNeoPortalUser() and not self.checkPassword( password ):
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="You don't have permissions to modify this comment." )
                if not self.isNeoPortalContentOwner() and not self.isNeoBoardManager():
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="You don't have permissions to modify this comment." )

            if manage_edit and creator:
                acl = getattr( self.getNeoBoard(), 'acl_users', None )
                if acl is None:
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="User Folder not found!" )          
                
                if acl.getUser( creator.strip() ) is None and not self.anonymousPostingAllowed():
                    return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='default', message="No such user!" )          
    
                self.setCreator( creator.strip() )
            else:
                self.setCreator( self.getAuthenticatedNeoPortalUser().getUserName() )
        else:
            if not creator:
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator', message="The field cannot be empty!" )            

            self.setCreator( creator )
            
            # manager? allow him/her to change the article's password
            if manage_edit and not self.checkPassword( password ):
                self.setPassword( password )
                                        
            if not password and not self.isNeoBoardManager():
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="The field cannot be empty!" )            

            if not self.checkPassword( password ) and not self.isNeoBoardManager():           
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='password', message="Password Mismatch!" )                            

        # empty email address not allowed
        if not self.emptyEmailAllowed():
            if not creator_email:
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="The field cannot be empty!" )            

            if not isEmailValid(creator_email.strip()):
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="Invalid Email Address!" )

        else:
            if creator_email.strip() and not isEmailValid(creator_email.strip()):
                return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='creator_email', message="Invalid Email Address!" )
                
                            
        # empty comment not allowed
        if not text:
            return self.manageNeoPortalError( obj=OBJ, REQUEST=REQUEST, field='text', message="The field cannot be empty!" )                
                
        self.setCreatorEmail(creator_email)
        
        # flush cache and reindex
        self.text = text
        self._updateNeoPortalContent()
        self.aq_inner.aq_parent._updateNeoPortalContent()

        # manager - hehe, we can track you down if we want to;-)
        self._setRemoteIP(REQUEST)
        return None

    security.declareProtected( NeoPortalView, 'manage_commentEditForm' )
    def manage_commentEditForm(self, REQUEST=None):
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        REQUEST.set('neoboard_form', 'neoboard_comment_edit_form')
        return self.neoboard_main
        return apply( self.neoboard_main, ( self, self._cookREQUEST( REQUEST ) ) )

    security.declareProtected( NeoPortalManageProperties, 'manage_editNeoBoardArticleComment' )
    def manage_editNeoBoardArticleComment( self, 
            creator='',
            password='',
            creator_email='',
            text='',
            REQUEST=None ):
        """ 
            Edit NeoBoard Article Comment
        """

        result = self._edit( creator=creator
                     , password=password
                     , creator_email=creator_email
                     , text=text
                     , manage_edit=1
                     , OBJ=self.aq_inner.aq_parent
                     , REQUEST=REQUEST )

        if result:
            return result
        
        if REQUEST is not None:
            parent = self.aq_inner.aq_parent        
            message = self.getNeoPortalMessage( message='Changes applied!' )
            query_str = self.getFullArticleQuery( REQUEST ) + '#' + self.getId()
        
            return REQUEST.RESPONSE.redirect( '%s?portal_status_message=%s&%s' % ( parent.absolute_url(), message, query_str ) )                    

    security.declarePrivate( '_cookContent' )
    def _cookContent(self, content, REQUEST=None, **kw):
        """
            Use parent's _cookContent
        """
        return self.aq_inner.aq_parent._cookContent( content, REQUEST )