#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardBase.py,v 1.19 2003/04/01 14:35:16 wankyu Exp $
Description

   NeoBoard Base Classes 
"""
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from OFS.DTMLMethod import DTMLMethod
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
from DocumentTemplate.DT_Util import html_quote
from OFS.SimpleItem import SimpleItem
from StructuredText import HTMLClass, DocumentClass, ST
from DateTime.DateTime import DateTime
from Acquisition import aq_parent, aq_base, aq_inner

try:
    from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2 as BaseFolder
    BTREEFOLDER_INSTALLED = 1
except ImportError:
    from OFS.Folder import Folder as BaseFolder
    BTREEFOLDER_INSTALLED = 0


from Products.NeoPortalLibrary.NP_Utils import removeTags, isEmailValid, encryptNeoPortalPassword
from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem
from Products.NeoPortalLibrary.NP_Content import NeoPortalContent
from Products.NeoPortalLibrary.NP_ElementContainer import NeoPortalElementContainer
from Products.NeoPortalLibrary.NP_Element import NeoPortalElement

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from Products.NeoPortalLibrary.NP_STDocument import NeoPortalStructuredDocumentHTMLNG

import re

MINIMAL_REQUEST_VARS = { 'b_start:int' : 0
                       , 'expand_all:int' : 0
                       , 'skey': ''
                       , 'reverse:int' : 0
                       , 'view_comment' : ''
                       , 'search_keywords' : ''
                       , 'search_fields_str' : ''}

class NeoBoardBaseItem( NeoPortalContent ):
    """
        NeoBoardBase Item Class
    """

    security = ClassSecurityInfo()
    
    ###################################################################################
    # 
    #       Public Methods - CMF DublinCore Compatible methods
    #
    ###################################################################################    
        
    security.declarePublic( 'isNeoBoardExpired' )
    def isNeoBoardExpired( self ):
        """
            Tell if NeoBoard has expired
        """
        
        return self.isObjectExpired()

    security.declarePublic( 'isNeoBoardEffective' )
    def isNeoBoardEffective( self, check_date=None ):
        """
            Tell if object is effective
        """
        
        return self.isObjectEffective()

    security.declarePrivate( '_cookREQUEST' )
    def _cookREQUEST( self, REQUEST, reset_form=0):
        """
            Cooks query string to be returned in REQUEST object
        """                 

        if reset_form and REQUEST.has_key('neoboard_form' ):
            REQUEST.set('neoboard_form', '' )
        
        return REQUEST   

    security.declarePrivate( 'manageNeoPortalError' )
    def manageNeoPortalError( self, obj=None, REQUEST=None, field='', message='', tup=() ):
        """
            Override NeoPortalTranslatableContent's method 
        """
        ret_message = self.getNeoPortalMessage( message, tup )
        
        try:
            if obj is not None:
                form = obj.neoboard_main
            else:
                form = self.neoboard_main
        except Exception, e:
            self.np_log( message="NeoBoardError: %s (%s) - %s" % ( field, ret_message, e ), summary='NeoBoardBase')
            raise 'NeoBoardError', '%s: %s' % ( field, ret_message )

        return NeoPortalContent.manageNeoPortalError( self, obj=obj, form=form, field=field, message=message, tup=tup, REQUEST=REQUEST )

    security.declarePublic( 'getFullArticleQuery' )
    def getFullArticleQuery( self, REQUEST ):
        """
            Return full article query string
        """

        r_vars = []
        
        for var in MINIMAL_REQUEST_VARS.keys():
            r_vars.append( var + '=' + str( getattr( REQUEST, var.split(':')[0],  MINIMAL_REQUEST_VARS[ var ] ) ) )

        return '&'.join( r_vars )
        
    security.declarePublic( 'getFullArticleQueryAsDictionary' )
    def getFullArticleQueryAsDictionary( self, REQUEST ):
        """
            Return full article query as dictionary
        """

        r_vars = {}
        
        for var in MINIMAL_REQUEST_VARS.keys():
            r_vars[ var.split(':')[0] ] = getattr( REQUEST, var.split(':')[0],  MINIMAL_REQUEST_VARS[ var ] )

        return r_vars

    security.declareProtected( NeoPortalView, 'CookedBody' )
    def CookedBody( self ):
        """
            Return cooked body 
        """ 
        if hasattr( self, 'neoboard_article' ):
            return self._cookContent( self.getRawNeoPortalContent() )
        else:
            return ''
                        
class NeoBoardBase( NeoPortalElementContainer, NeoBoardBaseItem  ):
    """
        NeoBoard Base
    """

    _neoboard_renderables = ('RAW', 'HTML','STX', 'DTML',)   
    _np_element_renumbering_allowed = 0
    
    _items_per_page = 30    # default num items per per page 
    
    _default_neoboard_format = 'STX'
    _default_neoboard_charset = 'UTF-8'
    _allowed_neoboard_charsets = ['ISO-8859-1', 'EUC-KR', 'KSC_5601',   
            'UTF-8', 'UTF-16', 'US-ASCII', 
            'ISO-8859-2', 'ISO-8859-3', 'ISO-8859-4', 
            'ISO-8859-5', 'ISO-8859-6', 'ISO-8859-7', 'ISO-8859-8', 
            'ISO-8859-9', 'ISO-2022-JP', 'ISO-2022-JP-2', 
            'ISO-2022-KR', 'windows-1252', 'GB2312', 'HZ-GB-2312',
            'SHIFT_JIS', 'JIS_X0208', 'BIG5', 'KOI8-R',
            ]
    
    _default_neoboard_language = 'en'
    
    _allowed_neoboard_languages = {
            'ab' : 'Abkhazian',
            'aa' : 'Afar',
            'af' : 'Afrikaans',
            'sq' : 'Albanian',
            'am' : 'Amharic',
            'ar' : 'Arabic',
            'hy' : 'Armenian',
            'as' : 'Assamese',
            'ay' : 'Aymara',
            'az' : 'Azerbaijani',
            'ba' : 'Bashkir',
            'eu' : 'Basque',
            'be' : 'Belarussian',
            'bn' : 'Bengali',
            'dz' : 'Bhutani',
            'bh' : 'Bihari',
            'bi' : 'Bislama',
            'br' : 'Breton',
            'bg' : 'Bulgarian',
            'my' : 'Burmese',
            'km' : 'Cambodian',
            'ca' : 'Catalan',
            'zh' : 'Chinese',
            'kw' : 'Cornish',
            'co' : 'Corsican',
            'hr' : 'Croatian',
            'cs' : 'Czech',
            'da' : 'Danish',
            'nl' : 'Dutch',
            'en' : 'English',
            'eo' : 'Esperanto',
            'et' : 'Estonian',
            'fo' : 'Faroese',
            'fj' : 'Fiji',
            'fi' : 'Finnish',
            'fr' : 'French',
            'fy' : 'Frisian',
            'gl' : 'Galician',
            'ka' : 'Georgian',
            'de' : 'German',
            'kl' : 'Greenlandic',
            'el' : 'Greek',
            'gn' : 'Guarani',
            'gu' : 'Gujarati',
            'ha' : 'Hausa',
            'he' : 'Hebrew',
            'hi' : 'Hindi',
            'hu' : 'Hungarian',
            'is' : 'Icelandic',
            'id' : 'Indonesian',
            'ia' : 'Interlingua',
            'ie' : 'Interlingue',
            'iu' : 'Inuktitut',
            'ik' : 'Inupiak',
            'ga' : 'Irish',
            'it' : 'Italian',
            'ja' : 'Japanese',
            'jw' : 'Javanese',
            'kn' : 'Kannada',
            'ks' : 'Kashmiri',
            'kk' : 'Kazakh',
            'ky' : 'Kirghiz',
            'rn' : 'Kirundi',
            'rw' : 'Kiyarwanda',
            'ko' : 'Korean',
            'ku' : 'Kurdish',
            'lo' : 'Laotian',
            'la' : 'Latin',
            'lv' : 'Latvian Lettish',
            'ln' : 'Lingala',
            'lt' : 'Lithuanian',
            'lb' : 'Luxemburgish',
            'mk' : 'Macedonian',
            'ms' : 'Malay',
            'ml' : 'Malayalam',
            'mg' : 'Malagasy',
            'mt' : 'Maltese',
            'gv' : 'Manx Gaelic',
            'mi' : 'Maori',
            'mr' : 'Marathi',
            'mo' : 'Moldavian',
            'mn' : 'Mongolian',
            'na' : 'Nauru',
            'ne' : 'Nepali',
            'se' : 'Northern S&aacute;mi',
            'no' : 'Norwegian',
            'oc' : 'Occitan',
            'or' : 'Oriya',
            'om' : 'Oromo',
            'ps' : 'Pashto',
            'fa' : 'Persian',
            'pl' : 'Polish',
            'pt' : 'Portuguese',
            'pa' : 'Punjabi',
            'qu' : 'Quechua',
            'rm' : 'Rhaeto-Romance',
            'ro' : 'Romanian',
            'ru' : 'Russian',
            'sm' : 'Samoan',
            'sg' : 'Sangho',
            'sa' : 'Sanskrit',
            'gd' : 'Scottish Gaelic',
            'sr' : 'Serbian',
            'sh' : 'Serbo-Croatian',
            'st' : 'Sesotho',
            'tn' : 'Setswana',
            'sn' : 'Shona',
            'sd' : 'Sindhi',
            'si' : 'Singhalese',
            'ss' : 'Siswati',
            'sl' : 'Slovenian',
            'sk' : 'Slovak',
            'so' : 'Somali',
            'es' : 'Spanish',
            'su' : 'Sudanese',
            'sw' : 'Swahili',
            'sv' : 'Swedish',
            'tl' : 'Tagalog',
            'tg' : 'Tajik',
            'ta' : 'Tamil',
            'tt' : 'Tatar',
            'te' : 'Telugu',
            'th' : 'Thai',
            'bo' : 'Tibetan',
            'ti' : 'Tigrinya',
            'to' : 'Tonga',
            'ts' : 'Tsonga',
            'tr' : 'Turkish',
            'tk' : 'Turkmen',
            'tw' : 'Twi',
            'ug' : 'Uigur',
            'uk' : 'Ukrainian',
            'ur' : 'Urdu',
            'uz' : 'Uzbek',
            'vi' : 'Vietnamese',
            'vo' : 'Volap&uuml;k',
            'cy' : 'Welsh',
            'wo' : 'Wolof',
            'xh' : 'Xhosa',
            'yi' : 'Yiddish',
            'yo' : 'Yorouba',
            'za' : 'Zhuang',
            'zu' : 'Zulu', 
            }
            
    _np_cataloggable_meta_types = [ 'NeoBoard Article'
                                  , 'NeoBoard Notice Article'
                                  , 'NeoBoard Article Attachment Element' ]
    
    _np_catalog_indexes = ( ('Title', 'TextIndexNG')
                           , ('meta_type', 'FieldIndex')
                           , ('getSortKey', 'FieldIndex')        
                           , ('getSortKeys', 'FieldIndex')       
                           , ('getThreadSortKey', 'FieldIndex')         
                           , ('isThreadParent', 'FieldIndex')                       
                           , ('CreationDate', 'FieldIndex')        
                           , ('Creator', 'FieldIndex')
                           , ('CreatorEmail', 'FieldIndex')
                           , ('getNeoBoardArticleCategory', 'FieldIndex')
                           , ('getNeoPortalContentSearchText', 'TextIndexNG')
                           , ('getInlineCommentsSearchText', 'TextIndexNG')               
                           , ('getInlineCommentCreators', 'TextIndexNG')               
                           , ('getAttachmentsSearchText', 'TextIndexNG')
                           , ('getNeoPortalReadCount', 'FieldIndex')
                           , ('getNeoPortalNumContentRatings', 'FieldIndex')
                           , ('getNeoPortalElementNumber', 'FieldIndex')
                           , ('isTempNeoBoardArticle', 'FieldIndex')
                           )            
            
    security = ClassSecurityInfo()
                   
    security.declarePublic( 'getDefaultNeoBoardCharset' )
    def getDefaultNeoBoardCharset( self ):
        """
            Return the default charset of the content
        """
        return self._default_neoboard_charset

    security.declarePublic( 'getDefaultNeoBoardFormat' )
    def getDefaultNeoBoardFormat( self ):
        """
            Return the default renderable format of the content
        """
        return self._default_neoboard_format

    security.declarePublic( 'getDefaultNeoBoardLanguage' )
    def getDefaultNeoBoardLanguage( self ):
        """
            Return the default language of the content
        """
        return self._default_neoboard_language

    security.declarePublic( 'getAllowedNeoBoardCharsets' )
    def getAllowedNeoBoardCharsets( self ):
        """
            Return the list of allowed charsets
        """
        
        return self._allowed_neoboard_charsets

    security.declarePublic( 'getAllowedNeoBoardLanguages' )
    def getAllowedNeoBoardLanguages( self ):
        """
            Return allowed languages as a dictionary
        """
        return self._allowed_neoboard_languages

    security.declarePublic( 'getAllowedNeoBoardLanguageKeys' )
    def getAllowedNeoBoardLanguageKeys( self ):
        """
            Return allowed language keys
        """
        return self.getAllowedNeoBoardLanguages().keys()

    security.declareProtected( NeoPortalManageProperties, 'setNeoBoardEnabled(' )
    def setNeoBoardEnabled( self, mode=1):
        """
            Enable or disable the NeoBoard instance
        """
        
        self.enabled = mode
        
    security.declarePublic( 'isNeoBoardEnabled' )
    def isNeoBoardEnabled( self ):
        """
            Tell if NeoBoard is enabled
        """
        
        if not self.enabled: return 0
        if not self.isNeoBoardExpired() and self.isNeoBoardEffective():
            return 1
        else:
            return 0        

    security.declarePublic( 'isNeoBoardViewable' )
    def isNeoBoardViewable( self ):
        """
            Tell if the NeoBoard instance is publicly viewable
        """
        
        if not self.isNeoBoardEnabled():
            return 0
        
        user = self.getAuthenticatedNeoPortalUser()
        if user is None or not hasattr( user, 'has_permission' ):
            return 0

        return user.has_permission( NeoPortalView, self )

    security.declarePublic( 'isNeoBoardEditable' )
    def isNeoBoardEditable( self ):
        """
            Tell if the authenticated user is authorized to edit this NeoBoard instance
        """
        
        user = self.getAuthenticatedNeoPortalUser()
        if user is None or not hasattr( user, 'has_permission' ):
            return 0

        return user.has_permission( NeoPortalAddPortalContent, self )            
    
    security.declarePublic( 'getNeoBoardCharset' )
    def getNeoBoardCharset( self ):
        """
            Return the charset of the NeoBoard instance
        """ 
        if hasattr( self, 'neoboard_charset' ):
            if not self.neoboard_charset in self.getAllowedNeoBoardCharsets():
                return self.getDefaultNeoBoardCharset()
            return self.neoboard_charset
        else:
            return self.getDefaultNeoBoardCharset()

    security.declarePublic( 'getNeoBoardRenderables' )
    def getNeoBoardRenderables( self ):
        """
            Return a tuple of renderable formats: HTML, DTML, STX, etc
        """
    
        return self._neoboard_renderables

    security.declarePublic( 'getItemsPerPage' )
    def getItemsPerPage( self ):
        """
            Tell how many items should appear on a list
        """

        if hasattr( self, 'items_per_page' ):
            return self.items_per_page

        try:
            return self.neoportal_properties.items_per_page                
        except:
            return self._items_per_page
            

    security.declarePrivate( 'PrincipiaSearchSource' )
    def PrincipiaSearchSource( self, REQUEST=None ):
        """
            Supports for full text search via ZCatalog indexing
        """
        delimiter = ' '
        return removeTags( delimiter.join( ( self.Title(), self.Description(), ) ) )

    security.declarePrivate(' getNeoBoardSkinFolder' )
    def getNeoBoardSkinFolder( self ):
        """
            Return the first element from the list of available NeoBoard Skin Folders: 
            
            multiple skin folders are meaningless
        """ 
        try:
            skin_folders = self.aq_inner.aq_parent.objectValues( 'NeoBoard Skin Folder', )
            if skin_folders and skin_folders[0].meta_type == 'NeoBoard Skin Folder':
                return skin_folders[0]
            else:
                return None
        except:
            return None

    security.declarePublic( 'getNeoBoardSkinId' )
    def getNeoBoardSkinId( self ):
        """
            Return the ID of the current skin
        """
        
        if hasattr( self, 'neoboard_skin' ):
            return self.neoboard_skin
        else:
            return 'default'

    security.declareProtected( NeoPortalManageProperties, 'setNeoBoardSkin' )
    def setNeoBoardSkin( self, skin ):
        """
            Set the current skin to the named one
        """ 
        self.neoboard_skin = skin
    
    security.declarePublic( 'getDefaultNeoBoardSkin' )
    def getDefaultNeoBoardSkin( self ):
        """
            Also used when the named skin is not available
        """
        
        return getattr( self.getNeoBoardSkinFolder(), 'default' )

    security.declarePrivate(' _getNeoBoardSkinElement' )
    def _getNeoBoardSkinElement( self, id ):
        """ 
            Return the requested skin element - _getNeoBoardSkinElement 
            basically does what _getViewFor does in CMF
        """
        
        skin = self.getNeoBoardSkin()

        if skin:
            elem = skin._getSkinElement( id )
            if elem:
                return elem
            else:
                # skin element not found in the current skin, try backups
                
                for s in skin.getBackupSkins():
                    elem = self.getNeoBoardSkin(skin=s)._getSkinElement( id )
                    if elem: 
                        return elem
        
        # even backup skins don't have the element, try default skin
        
        skin = self.getDefaultNeoBoardSkin()
        elem = skin._getSkinElement( id )
        
        if elem:
            return elem

        # damn! why would you want it all the way down here?
        
        raise 'NeoBoardError', "The default skin <em><b>%s</b></em> doesn't have an attribute '%s'!" % (skin.getId(), id)

    security.declarePublic( 'getNeoBoardSkins' )
    def getNeoBoardSkins( self ):
        """
            Return the list of available skins
        """
        return self.getNeoBoardSkinFolder().getNeoBoardSkins()

    security.declarePublic( 'getNeoBoardSkin' )
    def getNeoBoardSkin( self, skin=None ):
        """ 
            Return the named or current skin
        """
        
        skin_folder = self.getNeoBoardSkinFolder()
        
        if skin and hasattr(skin_folder, skin) and skin_folder._getOb( skin ).isSkinEnabled():
            return getattr(skin_folder, skin)
        
        skin = self.getNeoBoardSkinId()
        
        if skin and hasattr(skin_folder, skin) and skin_folder._getOb( skin ).isSkinEnabled():
            return getattr(skin_folder, skin)

        # skin not found, return default
        
        return self.getDefaultNeoBoardSkin()
        
class NeoBoardArticleBase( NeoPortalElementContainer, NeoPortalElement, NeoBoardBaseItem ):
    """
        NeoBoard Base Class
    """
        
    _np_inline_comment_meta_type = 'NeoBoard Article Comment'
    _np_element_renumbering_allowed = 0
    _np_top_most_element_container = 0
    
    _np_content_can_be_embedded = 0
    _np_content_can_embed = 1
        
    manage_options=(
        {'label':'Contents',   'action':'manage_main'},
        {'label':'View',       'action':'index_html'},
        {'label':'Security',   'action':'manage_access'},
        {'label':'Ownership',  'action':'manage_owner'},
    )    

    security = ClassSecurityInfo()

    def __init__( self, id ):
        
        if BTREEFOLDER_INSTALLED:
            BaseFolder.__init__( self, id=id )
        else:
            self.id = id
            
        self.text = self.cooked_text = ''

    security.declarePrivate( 'isTempNeoBoardArticle' )
    def isTempNeoBoardArticle( self ):
        """
            Tell if self is temporary object
        """
    
        if hasattr( aq_base( self ), '_is_temp_article' ) and self._is_temp_article: return 1
        return 0
        

    security.declarePublic( 'enumerateIndexes' )
    def enumerateIndexes( self ):
        """
            Call NeoBoard's
        """
        
        return self.getNeoBoard().enumerateIndexes()
        
    security.declarePublic( 'enumerateColumns' )
    def enumerateColumns( self ):
        """
            Call NeoBoard's
        """
        
        return self.getNeoBoard().enumerateColumns()
        
    security.declarePrivate( '_initIndexes' )
    def _initIndexes( self ):
        """
            Call NeoBoard's
        """
        
        return self.getNeoBoard()._initIndexes()

    security.declarePrivate( '_getObjectURL' )
    def _getObjectURL( self, ob ):
        """
            Call NeoBoard's
        """
        
        return self.getNeoBoard()._getObjectURL( ob )

    security.declarePrivate( 'getNeoPortalElementContainerCatalog' )
    def getNeoPortalElementContainerCatalog( self ):
        """
            Return Internal Catalog
        """         

        return self.getNeoBoard()
    
    security.declarePublic( 'searchResults' )   
    def searchResults( self, REQUEST=None, used=None, **kw ):
        """
            Call NeoBoard's
        """
        
        return self.getNeoBoard().searchResults( REQUEST=None, used=None, **kw  )
                
    security.declarePublic( 'inlineCommentsAllowed' )
    def inlineCommentsAllowed( self ):
        """
           Return NeoBoard's inlineCommentsAllowed
        """
        
        return self.getNeoBoard().inlineCommentsAllowed()        

    security.declarePublic( 'contentIconizationAllowed' )
    def contentIconizationAllowed( self ):
        """
           Return NeoBoard's contentIconizationAllowed
        """
        
        return self.getNeoBoard().contentIconizationAllowed()

    security.declarePublic( 'contentIconizationAllowed' )
    def spamBlockingEnabled( self ):
        """
           Return NeoBoard's spamBlockingEnabled
        """
        
        return self.getNeoBoard().spamBlockingEnabled()

    security.declarePublic( 'contentRatingAllowed' )        
    def contentRatingAllowed( self ):
        """
           Return NeoBoard's contentRatingAllowed
        """

        return self.getNeoBoard().contentRatingAllowed()
            
    security.declarePublic( 'getMinContentRatingValue' )        
    def getMinContentRatingValue( self ):
        """
           Return NeoBoard's getMinContentRatingValue
        """
        return self.getNeoBoard().getMinContentRatingValue()

    security.declarePublic( 'getMaxContentRatingValue' )        
    def getMaxContentRatingValue( self ):
        """
           Return NeoBoard's getMaxContentRatingValue
        """
        return self.getNeoBoard().getMaxContentRatingValue()
        
    security.declarePublic( 'getRawNeoPortalContentForReply' )        
    def getRawNeoPortalContentForReply( self ):
        """
            Return Raw Text stripped of embeds to be used in reply textarea
        """
        
        return self._stripInlineCommentMarkers( self.getRawNeoPortalContent() )

    security.declarePrivate( 'getInlineCommentButtonView' )        
    def getInlineCommentButtonView( self ):
        """
            Get inline comment form view
        """        
        return self.neoboard_comment_button    

    security.declarePrivate( '_cookContent' )
    def _cookContent( self, content, REQUEST=None, **kw):
        
        if REQUEST is None:
            if hasattr( self, 'REQUEST' ):
                REQUEST = self.REQUEST
        
        cooked_text = ''
        
        if 'DTML' in self.getNeoBoardRenderables() and self.Format() == 'DTML':
            if REQUEST is not None:
                content = DTMLMethod(content)
                content = apply(content.__call__, ( self, REQUEST))

            cooked_text = NeoPortalStructuredDocumentHTMLNG( content )
        
        elif 'STX' in self.getNeoBoardRenderables() and self.Format() == 'STX':
            cooked_text = NeoPortalStructuredDocumentHTMLNG( content )

        elif 'HTML' in self.getNeoBoardRenderables() and self.Format() == 'HTML':
            cooked_text = content

        else:
            # plain-text
            cooked_text = html_quote(content).replace('\n', '<br />' )

        return cooked_text
                
    security.declarePrivate( 'mailNeoBoardContent' )
    def mailNeoBoardContent( self, content_obj=None, is_comment=0, REQUEST=None ):
        """
            Mail Article or Comment
        """
        
        if REQUEST is None:
            REQUEST = self.REQUEST
        
        from_addr = "%s<%s>" % (self.getEmailFromName(), self.getEmailFromAddress())
        
        if content_obj is not None:
            # email reply or comment to the parent article poster

            if is_comment:
                subject = self.getNeoPortalMessage('Your Article Commented!' )            
            else:
                subject = self.getNeoPortalMessage('Your Article Replied!' ) + ' - ' + content_obj.Title()

            body = getattr(content_obj.__of__(self), 'neoboard_article_mail_view' )
            body = apply(body, (content_obj, REQUEST))

            creator_email = self.CreatorEmail()
            reply_email = content_obj.CreatorEmail()
            
            if isEmailValid( creator_email ):
                if isEmailValid(reply_email):
                    self.sendmail(from_addr=from_addr, to_addr=creator_email, reply_addr=reply_email, subject=subject, body=body, html=1)
                else:
                    self.sendmail(from_addr=from_addr, to_addr=creator_email, reply_addr='', subject=subject, body=body, html=1)
        else:
            # email self to NeoBoard Managers

            if self.getNeoBoard().getNeoBoardManagers() > 0:
                if is_comment:
                    subject = self.getNeoPortalMessage( 'NeoBoard Article Comment' )
                else:
                    subject = self.getNeoPortalMessage( 'NeoBoard Article' ) + ' - ' + self.Title()

                body = getattr( self, 'neoboard_article_mail_view' )

                body = apply(body, ( self, REQUEST))
                
                managers = self.getNeoBoard().getNeoBoardManagers()
                if len(managers) > 0:
                    for manager_email in managers.values():
                        if manager_email and isEmailValid(manager_email):
                            self.sendmail(from_addr=from_addr, to_addr=manager_email, reply_addr='', subject=subject, body=body, html=1)        

    def PUT( self, REQUEST, RESPONSE):
        """ 
            Handle FTP/WebDav PUT requests 
        """

        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        content = REQUEST['BODY']
        self.setRawNeoPortalContent(content.read())
        self._updateNeoPortalContent()
        RESPONSE.setStatus(204)
        return RESPONSE

    manage_FTPput = PUT
    manage_FTPget = NeoPortalContent.getRawNeoPortalContent 

InitializeClass( NeoBoardBase )
InitializeClass( NeoBoardArticleBase )