#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardSkinTool.py,v 1.7 2002/12/14 18:38:54 wankyu Exp $
Description

    NeoBoard SkinTool 
"""

import os, sys, re,types
from Globals import package_home, InitializeClass

this_module=sys.modules[__name__]

SKINS_FILE_PATH = package_home(globals())+ '/neoboard_skins/'
# in case called by other module when testing
SKINS_FILE_PATH = SKINS_FILE_PATH.replace(this_module.__name__ + os.sep, '')

if __name__ == '__main__':
    SKINS_FILE_PATH = SKINS_FILE_PATH.replace('__main__', '')

from AccessControl import ClassSecurityInfo 
from Globals import DTMLFile
from OFS.Folder import Folder
from OFS.DTMLMethod import DTMLMethod
from Acquisition import aq_base
from Products.NeoPortalLibrary.NP_Utils import addImage, setFormError, setFormMsg
from Products.NeoBoard.NeoBoardUtils import addObject

from Products.NeoPortalLibrary.NP_BaseItems import NeoPortalBaseItem

######################################## NEOPORTAL PERMISSIONS ##########################################################################
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalView, NeoPortalAccessContentsInformation
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddFolders, NeoPortalListUndoableChanges
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalViewManagementScreens, NeoPortalManageProperties
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalManagePortal, NeoPortalAddPortalContent
from Products.NeoPortalLibrary.NP_Permissions import NeoPortalAddPortalFolders, NeoPortalModifyPortalContent, NeoPortalReviewPortalContent
######################################## NEOPORTAL PERMISSIONS ##########################################################################

from os import listdir

StringType=type('')
ListType=type([])

manage_addNeoBoardSkinFolderForm = DTMLFile( 'dtml/SkinFolderAdd', globals() )

def manage_addNeoBoardSkinFolder( self
            , id=''
            , title=''
            , REQUEST=None ):
    """ 
        Add NeoBoard Skin Folder 
    """
        
    skin_folder = NeoBoardSkinFolder( id=id, title=title )
    skin_folder.id = id
    self._setObject( id, skin_folder )
    
    skin_folder = self._getOb( id )
    skin_folder._initSkins()
        
    if REQUEST is not None:
        return self.manage_main( self, REQUEST, update_menu=1 )

class NeoBoardSkinFolder( Folder, NeoPortalBaseItem ):
    """
        NeoBoard SkinFolder
    """
    
    meta_type='NeoBoard Skin Folder'
    icon='misc_/NeoBoard/neoboard_skinfolder_icon'

    meta_types=(
        {'name': 'NeoBoard Skin', 'action': 'manage_addNeoBoardSkinForm'},        
    )

    all_meta_types = meta_types
    
    manage_options=(
        {'label':'Contents',       'action':'manage_main'}, 
        {'label':'Properties', 'action':'manage_editNeoBoardSkinFolderForm'},
        {'label':'Reset Skins', 'action':'manage_resetNeoBoardSkinFolder'},
        {'label':'Security',   'action':'manage_access'},
        {'label':'Ownership',  'action':'manage_owner'},
        {'label':'Undo',  'action':'manage_undoForm'},
    )   
    
    security = ClassSecurityInfo()
    
    security.declareProtected( NeoPortalManageProperties , 'manage_editNeoBoardSkinFolderForm' )
    manage_editNeoBoardSkinFolderForm = DTMLFile('dtml/SkinFolderEdit', globals())
    
    def __init__(self, id, title=None):
        self.id = id
        self.title=title
        if not title:
            self.title = id
    
    security.declarePrivate( '_get_id' )
    def _get_id(self, id):
        """
            Conjure up a temporary filename for an object being cut and pasted
        """
        
        return id + '.TEMP'
    
    security.declarePublic( 'getNeoBoardSkins' )
    def getNeoBoardSkins( self ):
        return self.objectValues( NeoBoardSkin.meta_type )

    security.declareProtected( NeoPortalManageProperties, 'manage_resetNeoBoardSkinFolder')
    def manage_resetNeoBoardSkinFolder(self, REQUEST=None):
        """
            Reset Default Skins
        """
        
        return self.manage_editNeoBoardSkinFolder(title=self.title, reset_default_skins=1, REQUEST=REQUEST)
    
    security.declareProtected( NeoPortalManageProperties, 'manage_editNeoBoardSkinFolder')
    def manage_editNeoBoardSkinFolder(self
            , title=''
            , reset_default_skins=0
            , REQUEST=None):
                
        """ 
            Edit NeoBoard Skin Folder 
        """

        if not title:
            title = id
            
        self.title = title
        
        if reset_default_skins:    
                self._initSkins()
            
        if REQUEST is not None:
            return self.manage_main( self, REQUEST, update_menu=1 )
    
    security.declarePrivate( '_initSkins' )
    def _initSkins( self ):
        """
            Initialize skins in filesystem
        """
        
        default_skin = 'default'
        
        # deal with the default skin first
        
        if os.path.isdir(SKINS_FILE_PATH + default_skin) and os.path.isfile(SKINS_FILE_PATH + default_skin + '/skin_name.txt'):
            skin = NeoBoardSkin(id = default_skin)
            skin.enabled = 1
            skin.addSkinFromFile(SKINS_FILE_PATH + default_skin + '/')
            if hasattr(self, skin.id):
                self.manage_delObjects([skin.id,])
                
            self._setObject(default_skin, skin)
        else:
            raise 'NeoBoardError', 'Default Filesystem Skin Path not found: %s' % (SKINS_FILE_PATH + default_skin + '/',)
        
        for fname in listdir(SKINS_FILE_PATH):
            if fname == default_skin:
                continue
            if os.path.isdir(SKINS_FILE_PATH + fname) and os.path.isfile(SKINS_FILE_PATH + fname + '/skin_name.txt'):
                skin = NeoBoardSkin(id = fname)
                skin.enabled = 1
                skin.addSkinFromFile(SKINS_FILE_PATH + fname + '/')
                if hasattr(self, skin.id):
                    self.manage_delObjects([skin.id,])
                    
                self._setObject(fname, skin)

def manage_addNeoBoardSkin( self
        , id
        , inherit_from='default'
        , title='' 
        , enabled=0
        , creators=[]
        , backup_skins=[]
        , REQUEST=None ):
    """ 
        Add NeoBoard Skin
    """
    
    pointer = self.manage_copyObjects( [inherit_from,] )
    self.manage_pasteObjects( pointer )
    self.manage_renameObject( inherit_from + '.TEMP', id )

    skin = getattr( self, id )
    
    skin.title = title
    
    result = skin.setBackupSkins( backup_skins )
    if result is not None:
        self.manage_delObjects( [id,] )
        return self.manage_addNeoBoardSkinForm( setFormError( REQUEST, 'backup_skins', result ) )
    
    skin.setSkinCreators( creators )
    skin.enabled = enabled

    if REQUEST is not None:
        return self.manage_main( self, REQUEST, update_menu=1 )
        
manage_addNeoBoardSkinForm = DTMLFile( 'dtml/SkinAdd', globals() )        
        
class NeoBoardSkin( Folder, NeoPortalBaseItem ):
    """
        NeoBoard Skin
    """

    meta_type='NeoBoard Skin'
    icon='misc_/NeoBoard/neoboard_skin_icon'

    manage_options=(
        {'label':'Contents',   'action':'manage_main'},
        {'label':'Properties', 'action':'manage_editForm'},
        {'label':'Security',   'action':'manage_access'},
        {'label':'Ownership',  'action':'manage_owner'},
        {'label':'Undo',  'action':'manage_UndoForm'},
    )

    meta_types=(
        {'name': 'DTML Document', 'action': 'manage_addProduct/OFSP/documentAdd'},
        {'name': 'DTML Method', 'action': 'manage_addProduct/OFSP/methodAdd'},
        {'name': 'File', 'action': 'manage_addProduct/OFSP/fileAdd'},
        {'name': 'Folder', 'action': 'manage_addProduct/OFSP/folderAdd'},
        {'name': 'Image', 'action': 'manage_addProduct/OFSP/imageAdd'}, 
        {'name': 'Page Template', 'action': 'manage_addProduct/PageTemplates/manage_addPageTemplateForm'},          
    )

    all_meta_types = meta_types
    
    security = ClassSecurityInfo()  
        
    security.declareProtected( NeoPortalManageProperties, 'manage_editForm' )
    manage_editForm = DTMLFile( 'dtml/SkinEdit', globals() )   

    def __init__( self, id ):
        self.id = id

    security.declarePrivate( 'addSkinFromFile' )
    def addSkinFromFile(self, dir):
        """
            Used when the whole Skin Folder gets created
        """
        
        if os.path.isfile(dir + 'skin_name.txt'):
            lines = open(dir + 'skin_name.txt').readlines()
            if len(lines) > 0 and lines[0].strip():
                self.title = lines[0].strip()
            else:
                self.title = self.id

        for fname in listdir(dir):
            id = title = os.path.splitext(fname)[0]
            
            addObject(self, id, title, dir + fname)

        if os.path.isdir(dir + 'icons'):
            self.manage_addFolder('icons','Skin Icons')
            icon_folder = self._getOb('icons')

            for fname in listdir(dir + 'icons'):
                id = title = os.path.basename(fname) # should retain ext for compatibility with CMF
                addObject(icon_folder, id, title, dir + 'icons/' + fname)

        if os.path.isdir(dir + 'images'):
            self.manage_addFolder('images','Skin Images')
            images_folder = self._getOb('images')
    
            for fname in listdir(dir + 'images'):
                id = title = os.path.basename(fname) # should retain ext for compatibility with CMF
                addObject(images_folder, id, title, dir + 'images/' + fname)

        if os.path.isdir(dir + 'templates'):
            self.manage_addFolder('templates','Skin Templates')                    
            template_folder = self._getOb('templates')
    
            for fname in listdir(dir + 'templates'):
                id = title = os.path.splitext(fname)[0]
                addObject(template_folder, id, title, dir + 'templates/' + fname)
                
        if os.path.isdir(dir + 'forms'):
            self.manage_addFolder('forms','Skin Forms')
            icon_folder = self._getOb('forms')

            for fname in listdir(dir + 'forms'):
                id = title = os.path.splitext(fname)[0]
                addObject(icon_folder, id, title, dir + 'forms/' + fname)                

        if os.path.isdir(dir + 'scripts'):
            self.manage_addFolder('scripts','Skin Scripts')
            icon_folder = self._getOb('scripts')

            for fname in listdir(dir + 'scripts'):
                id = title = os.path.splitext(fname)[0]
                addObject(icon_folder, id, title, dir + 'scripts/' + fname)                
    
        self.creators=[]
        self.backup_skins=[]

    security.declareProtected( NeoPortalManageProperties, 'manage_editSkin' )
    def manage_editSkin(self
                , title=''
                , enabled=0
                , creators=[]
                , backup_skins=[]
                , REQUEST=None):
            
        """ Edit NeoBoard Skin"""
    
        self.title = title
                
        result = self.setBackupSkins( backup_skins ) 
        if result is not None:
            return self.manage_editForm( setFormError( REQUEST, 'backup_skins', result ) )    
        
        self.setSkinCreators( creators )
        self.enabled=enabled

        if REQUEST is not None:
            return self.manage_editForm( setFormMsg( REQUEST, 'Changes applied!' ) )        

    security.declareProtected( NeoPortalManageProperties, 'setBackupSkins' )
    def setBackupSkins(self, skins):
        """
            Backup Skins are used when the current Skin doesn't have a requested object
        """
        skin_ids = [ skin.getId() for skin in self.getNeoBoardSkins() ]
        
        if type( skins ) == ListType:
            for skin in skins:
                if not skin in skin_ids:
                    return "Skin <em>%s</em> not found in avaliable skins: %s" % ( skin, ','.join(skin_ids) )
            self.backup_skins = skins
        else:
            skins_list = skins.split('\n')
            for skin in skins_list:
                if not skin in skin_ids:
                    return "Skin <em>%s</em> not found in avaliable skins: %s" % ( skin, ','.join(skin_ids) )
            self.backup_skins = skins_list
        
        return None

    security.declarePublic( 'getBackupSkins' )
    def getBackupSkins( self ):
        """
            Return the list of backup Skins
        """
        
        return self.backup_skins

    security.declareProtected( NeoPortalManageProperties, 'setSkinCreators' )
    def setSkinCreators( self, creators ):
        """
            Set the list of Skin creators
        """
        
        if type( creators ) == ListType:
            self.creators = creators
        else:
            self.creators = creators.split('\n')

    security.declarePublic( 'getSkinCreators' )
    def getSkinCreators( self ):
        """
            Return the list of Skin creators
        """
        
        return self.creators

    security.declarePrivate( '_getSkinElement' )
    def _getSkinElement(self, id):
        """
            Return the requested element
        """
        skin_folder = self.aq_inner.aq_parent
        skin = skin_folder._getOb(self.getId())
        
        try:
            elem = getattr(aq_base(skin.icons), id)
        except AttributeError:
            try:
                elem = getattr(aq_base(skin.images), id)
            except AttributeError:
                try:
                    elem = getattr(aq_base(skin.templates), id)
                except AttributeError:                    
                    try:
                        elem = getattr(aq_base(skin.scripts), id)                    
                    except AttributeError:
                        try:                        
                            elem = getattr(aq_base(skin.forms), id)                                            
                        except AttributeError:
                            elem = None                    
        return elem
    
    security.declarePublic( 'isSkinEnabled' )
    def isSkinEnabled( self ):
        return self.enabled
        
InitializeClass( NeoBoardSkin )
