#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: NeoBoardUtils.py,v 1.4 2002/12/13 01:42:43 wankyu Exp $
Description

    NeoBoard Utilities
"""

from Globals import DTMLFile
from OFS.Image import Image
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from ZPublisher.Converters import get_converter
from AccessControl import ClassSecurityInfo

from Products.NeoPortalLibrary.NP_Utils import addDTML, addZPT, addImage, addPyScript

import os, os.path, string, types, urllib, re

class NeoBoardProperties ( SimpleItem, PropertyManager ):
    """
        Adapted from CMF's FSPropertiesObjects to use CMF skins' file system properties file
    """

    meta_type = 'NeoBoard Properties Object'
    
    icon = 'misc_/OFSP/dtmldoc.gif'

    manage_options = ({'label':'View', 'action':'manage_main'},)

    security = ClassSecurityInfo()
    
    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile( 'dtml/NeoBoardPropertiesView', globals())

    security.declarePrivate('manage_addProperty',
                            'manage_editProperties',
                            'manage_delProperties',
                            'manage_changeProperties',
                            'manage_propertiesForm',
                            'manage_propertyTypeForm',
                            'manage_changePropertyTypes',)

    security.declarePrivate( 'parseFile' )
    def parseFile( self, fname ):
        """
            Read the data from the give file.
        """

        file = open( fname, 'rb' )
        try:
            lines = file.readlines()
        finally:
            file.close()

        map = []
        lino=0

        for line in lines:

            lino = lino + 1
            line = line.strip()

            if not line or line[0] == '#':
                continue

            #try:
            propname, proptv = line.split(':')
            proptype, propvstr = proptv.split( '=', 1 )
            propname = propname.strip()
            proptype = proptype.strip()
            propvstr = propvstr.strip()
            converter = get_converter( proptype, lambda x: x )
            propvalue = converter( propvstr.strip() )
            setattr(self, propname, propvalue)
            map.append( {'id':propname,
                        'type':proptype,
                        'mode':'',
                        'default_value':propvalue,
                        } )
            #except:
            #    raise ValueError, ( 'Error processing line %s of %s:\n%s'
            #                      % (lino,fname,line) )
        self._properties = tuple(map)

def addObject( obj, id, title, fname, overwrite=0 ):
    """
        Add an object from filesystem
    """
    
    if(fname.endswith('.dtml')):
        addDTML(obj, id, title, fname, overwrite)
    elif(fname.endswith('.zpt') or fname.endswith('.pt')):
        addZPT(obj, id, title, fname, overwrite)    
    elif(fname.endswith('.py')):        
        addPyScript(obj, id, fname, overwrite)  
    elif(fname.endswith('.props')):        
        addNeoBoardProperties(obj, id, title, fname, overwrite)          
    elif(fname.endswith('.gif') or fname.endswith('.jpg') or fname.endswith('.png')):
        addImage(obj, id, title, fname, overwrite)
        
def addNeoBoardProperties( obj, id, title, fname, overwrite=0 ):
    """
        Add a NeoBoard Properties Object from filesystem
    """
    
    if overwrite:
        try:
            obj.manage_delObjects([id + '._BACKUP',])
        except:
            pass
        try:
            obj._setObject(id + '._BACKUP', getattr(obj, id))
        except:
            pass

        try:
            obj.manage_delObjects([id,])
        except:
            pass
            
    prop = NeoBoardProperties(id=id)
    prop.title = title
    prop.parseFile(fname)
    
    obj._setObject(id, prop)
    return getattr(obj,id)