#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: __init__.py,v 1.8 2003/03/27 11:08:01 wankyu Exp $
Description

    NeoBoard is a threaded web message boards originally developed in Perl, then ported to PHP. 
    
    Now, NeoBoard is now reborn as a Zope product.
    
    NeoBoard can be used either as a standalone Zope product or as CMF/Plone product.
    
"""

from Globals import InitializeClass, ImageFile
from AccessControl import ClassSecurityInfo

from Products.NeoBoard.NeoBoard import NeoBoard, manage_addNeoBoardForm, manage_addNeoBoard, stylesheet, getNeoBoardInitSkinFolder
from Products.NeoBoard.NeoBoardSkinTool import NeoBoardSkinFolder, manage_addNeoBoardSkinFolderForm, manage_addNeoBoardSkinFolder
from Products.NeoBoard.NeoBoardArticle import NeoBoardArticle, addNeoBoardArticle

from Products.NeoBoard.PloneBatch import Batch as BaseBatch
            
try:    
    from Products.CMFPlone.PloneUtilities import IndexIterator    
except:

    from AccessControl import allow_module 
    allow_module('Products.NeoBoard')

    from Products.NeoPortalLibrary.NP_Utils import IndexIterator
    
neoboard_globals = globals()

BaseBatch._security = sec = ClassSecurityInfo() 
sec.declareObjectPublic() 
sec.setDefaultAccess( 1 ) 
sec.apply( BaseBatch )
InitializeClass( BaseBatch )

IndexIterator._security = sec = ClassSecurityInfo() 
sec.declareObjectPublic() 
sec.setDefaultAccess( 1 ) 
sec.apply( IndexIterator )
InitializeClass( IndexIterator )

def initialize( context ):
    context.registerClass(
            NeoBoard
          , constructors = ( manage_addNeoBoardForm, manage_addNeoBoard )
          , permission = 'Add NeoBoard'
          , icon = 'www/neoboard.gif'
        )

    context.registerClass(
            NeoBoardSkinFolder
          , constructors = ( manage_addNeoBoardSkinFolderForm, manage_addNeoBoardSkinFolder )
          , permission = 'Add NeoBoard Skin Folder'
          , icon = 'www/neoboard_skinfolder.gif'
        )

    try:
        from Products.CMFCore import utils
    except ImportError:
        # CMF not available
        pass
    else:
        # CMF found
        from Products.CMFCore.DirectoryView import registerDirectory

        board_skins = ( 'neoboard_skins', )
        for skin in board_skins:
            registerDirectory( skin, globals() )
            
        from Products.CMFCore import CMFCorePermissions, utils
        from Products.NeoBoard import CMFNeoBoard
        import sys
      
        this_module = sys.modules[ __name__ ]
        contentClasses=( CMFNeoBoard.CMFNeoBoard, )
        contentConstructors=( CMFNeoBoard.addCMFNeoBoard, )
        
        z_bases = utils.initializeBasesPhase1( contentClasses, this_module )

        utils.initializeBasesPhase2( z_bases, context )
        
        utils.ContentInit(
                'CMFNeoBoard' 
              , content_types=contentClasses
              , permission=CMFCorePermissions.AddPortalContent
              , extra_constructors=contentConstructors
              , fti=CMFNeoBoard.factory_type_information
            ).initialize(context)

methods={ 
        'getNeoBoardMailHosts': NeoBoard.getMailHosts,
        'getNeoBoardStyleSheet': stylesheet,
        'getNeoBoardInitSkinFolder': getNeoBoardInitSkinFolder,
        'addNeoBoardArticle': addNeoBoardArticle,
        'manage_addNeoBoardSkinFolderForm': NeoBoardSkinTool.manage_addNeoBoardSkinFolderForm,
        'manage_addNeoBoardSkinFolder': NeoBoardSkinTool.manage_addNeoBoardSkinFolder,
        'manage_addNeoBoardSkinForm': NeoBoardSkinTool.manage_addNeoBoardSkinForm,
        'manage_addNeoBoardSkin': NeoBoardSkinTool.manage_addNeoBoardSkin,
}

misc_={
        'neoboard_icon': ImageFile( 'www/neoboard.gif',globals() ),
        'neoboard_article_icon': ImageFile( 'www/neoboard_article.gif',globals() ),
        'neoboard_article_comment_icon': ImageFile( 'www/neoboard_article_comment.gif',globals() ),       
        'neoboard_skinfolder_icon': ImageFile( 'www/neoboard_skinfolder.gif',globals() ),
        'neoboard_skin_icon': ImageFile( 'www/neoboard_skin.gif',globals() ),
}