#######################################################################################
#                                                                                     #
# Copyright (c) 2002 Wankyu Choi. All Rights Reserved.                                #
#                                                                                     #
#   A copy of the GPL should accompany this distribution. Redistributions             #
#   in source code must retain the above copyright notice, accompanied license,       #
#   and the following disclaimer.                                                     #
#                                                                                     #
#   THIS SOFTWARE IS PROVIDED BY WANKYU CHOI ``AS IS'' AND ANY                        #
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE               #
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR                #
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WANKYU CHOI OR ITS                     #
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,                      #
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT                  #
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF                  #
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND               #
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,                #
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT                #
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF                #
#   SUCH DAMAGE.                                                                      #
#                                                                                     #
#######################################################################################

"""
$Id: test_NeoBoard.py,v 1.7 2003/04/01 14:35:16 wankyu Exp $
Description

 Test NeoBoard
"""

import Zope
from ZPublisher.HTTPRequest import HTTPRequest, HTTPResponse, FileUpload

from sys import stdin
from os import environ
environ['SERVER_NAME'] = 'NeoBoard Test Server'
environ['SERVER_URL'] = 'http://www.neoboard.net/'
environ['SERVER_PORT'] = '8080'
REQUEST = HTTPRequest(stdin, environ, HTTPResponse())    

REQUEST['SERVER_URL'] = 'http://www.neoboard.net'

from unittest import TestCase, TestSuite, makeSuite, main
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from AccessControl.SecurityManager import setSecurityPolicy

from Products.CMFCore.tests.base.testcase import SecurityRequestTest, RequestTest

try:
    from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2 as BaseFolder
except ImportError:
    from OFS.Folder import Folder as BaseFolder

from Globals import package_home
from Products.MailHost.MailHost import MailHost, manage_addMailHost
from Products.NeoBoard.NeoBoard import NeoBoard
from Products.NeoBoard.NeoBoardArticle import NeoBoardArticle, addNeoBoardArticle
from Products.NeoBoard.NeoBoardArticleComment import NeoBoardArticleComment, NEOBOARD_REQUEST_COMMENT_PREFIX, addNeoBoardArticleComment
from Products.NeoBoard.NeoBoardArticleAttachment import NeoBoardArticleAttachment
from Products.NeoBoard.NeoBoardSkinTool import NeoBoardSkinFolder

import os, re

ARCHIVE_FILE_PATH = package_home(globals())+ 'test_archive/'
ARCHIVE_FILE_PATH = ARCHIVE_FILE_PATH.replace('__main__', '')

ARCHIVE_FILE_PATH2 = package_home(globals())+ 'test_archive2/'
ARCHIVE_FILE_PATH2 = ARCHIVE_FILE_PATH2.replace('__main__', '')

ARCHIVE_MARKER = '.__neoboard_archive_path__'

BaseTest = SecurityRequestTest

class Null:
    def __init__( self, **kw ):
        pass

    def __call__( self, REQUEST=None, RESPONSE=None, **kw ):
        return 'RENDERED'

class NeoBoardTests( BaseTest ):
    
    #setUp/tearDown
    def setUp( self ):
        """
            Add root Folder, SkinFolder, NeoBoard, Article with FileSystem Attachments
        """
        
        BaseTest.setUp(self)
                
        # add a test folder
        self.folder = BaseFolder(id='testFolder')
        self.folder.REQUEST = REQUEST

        # add test mailhost
        manage_addMailHost(self.folder, id='MailHost')
        
        # add skin
        skin = NeoBoardSkinFolder(id='SkinFolder')
        skin.REQUEST = REQUEST
        skin._initSkins()
        self.folder._setObject('NeoBoardSkinFolder', skin)
        
        # add board
        board = NeoBoard(id='NeoBoard')
        board.REQUEST = REQUEST
        board.setTitle('Test Board')
        board.setNeoBoardEnabled(1)
        self.folder._setObject('NeoBoard', board)
        self.getNeoBoard()._addCatalog()
        self.getNeoBoard().manage_editNeoBoard(title='testBoard'
                            , description='This is a test board.'
                            , neoboard_skin='default'
                            , mailhost='MailHost'
                            , use_filesystem=1
                            , archive_path=ARCHIVE_FILE_PATH
                            , num_attachments=5
                            , allow_inline_comments=1
                            , format='DTML'
                            , charset='euc_kr'
                            , auto_approve_article=1
                            , check_filepath=0
                            , enabled=1
                            , REQUEST=None)

        # add article
        addNeoBoardArticle( self.getNeoBoard(),
                creator='wankyu',
                password='12345',
                creator_email='wankyu@neoqst.com',
                title='This is a test article.', 
                text='What is this for?\n\n Just a test.\n\nComments?\n\n',
                category='Test',
                attachments=[],
                mail_reply=0,
                check_filepath=0,
                REQUEST=REQUEST)  
             
        # upload some files
        file_1 = open('./test_file.txt', 'rb')  # text
        file_2 = open('./test_file2.txt', 'rb') # text
        file_3 = open('./test_file3.gif', 'rb') # binary
  
        class F:
            def __init__(self, file):
                self.file = file
                self.filename = file.name
                self.headers = {}
  
        file_1 = FileUpload(F(file_1))
        file_2 = FileUpload(F(file_2))
        file_3 = FileUpload(F(file_3))

        file_1 = {'title': 'Test1', 'description': 'Test File 1', 'file': file_1}
        file_2 = {'title': 'Test2', 'description': 'Test File 2', 'file': file_2}
        file_3 = {'title': 'Test3', 'description': 'Test File 3', 'file': file_3}
    
        attachments=[file_1, file_2, file_3]

        article = self.getNeoBoardArticle()

        addNeoBoardArticleComment( article
                                 , p_index=1
                                 , creator='wankyu'
                                 , password='12345'
                                 , creator_email='wankyu@neoqst.com'
                                 , text='This is an inline comment!'
                                 , REQUEST=REQUEST)

        self.dummyfy()

        article._edit( creator='wankyu',
            password='12345',
            creator_email='wankyu@neoqst.com',
            title='Modified with attachments', 
            text="We got attachments!\n\n Ain't it cool?",
            category='TEST',
            attachments=attachments,
            check_filepath=0,
            REQUEST=REQUEST)

    def getSkinFolder( self ):
        skinfolder = self.folder._getOb('NeoBoardSkinFolder')
        skinfolder = skinfolder.__of__( self.folder )
        return skinfolder
        
    def getMailHost( self ):
        mailhost = self.folder._getOb('MailHost')
        mailhost = mailhost.__of__( self.folder )
        return mailhost
        
    def getNeoBoard( self ):
        neoboard = self.folder._getOb('NeoBoard')
        neoboard = neoboard.__of__( self.folder )
        return neoboard
                
    def getNeoBoardArticle( self ):
        article = self.getNeoBoard()._getOb('a_1')
        article = article.__of__( self.getNeoBoard() )
        return article

    def getComment( self ):
        article = self.getNeoBoardArticle()
        return article.getInlineComment( NEOBOARD_REQUEST_COMMENT_PREFIX+'1' )

    def getCatalog( self ):
        catalog = self.getNeoBoard()._getOb('NeoBoardCatalog_1')        
        catalog = catalog.__of__( self.getNeoBoard() )
        return catalog 
        
    def tearDown(self):
        
        BaseTest.tearDown(self)
        
        article = self.getNeoBoardArticle()      
        try:
            self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH
            article.getAttachment(id=1)._resetFilePath()
            article.getAttachment(id=2)._resetFilePath()
            article.getAttachment(id=3)._resetFilePath()               
    
            article.getAttachment(id=1)._delete_file()
            article.getAttachment(id=2)._delete_file()
            article.getAttachment(id=3)._delete_file()
    
            self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH2
            article.getAttachment(id=1)._resetFilePath()
            article.getAttachment(id=2)._resetFilePath()
            article.getAttachment(id=3)._resetFilePath()               
    
            article.getAttachment(id=1)._delete_file()
            article.getAttachment(id=2)._delete_file()
            article.getAttachment(id=3)._delete_file()
            
            try:
                os.unlink(ARCHIVE_FILE_PATH  + ARCHIVE_MARKER)
                os.unlink(ARCHIVE_FILE_PATH2 + ARCHIVE_MARKER)
            except:
                pass
                
            os.removedirs(ARCHIVE_FILE_PATH + article.getId())
            os.removedirs(ARCHIVE_FILE_PATH2+ article.getId())            
            
        except:
            pass

    def dummyfy( self ):

        def dummy( **kw ): return ''
        def dummy2( self=self ): return self
        
        self.getNeoBoard().isNeoBoardManager = dummy
        
        self.getNeoBoard().manageNeoPortalError = dummy
        self.getNeoBoardArticle().manageNeoPortalError = dummy
        self.getComment().manageNeoPortalError = dummy
        
        self.getNeoBoard().getPhysicalRoot = dummy2   
        self.getNeoBoardArticle().getPhysicalRoot = dummy2     
        self.getComment().getPhysicalRoot = dummy2
        
        self.getNeoBoardArticle().getInlineCommentButtonView = Null
        self.getNeoBoardArticle().getInlineCommentView = Null
        self.getNeoBoardArticle().getLostInlineCommentView = Null

    def test_NeoBoard( self ):
        self.failUnless( self.getNeoBoard().Title() == 'testBoard',  "Can't instantiate NeoBoard!")        

    def test_NeoBoardArticle( self ):
        self.failUnless( self.getNeoBoardArticle().Creator() == 'wankyu',  "Can't instantiate NeoBoard Article!")  

        result = self.getNeoBoardArticle()._edit( creator='wankyu'
                     , password='123'
                     , creator_email='wankyu2@neoqst.com'
                     , title='Changed Title'
                     , text='Changed Text'
                     , category='Changed Category'
                     , REQUEST=REQUEST )              

        self.failIf( result is None ,  "Article password check not working! - %s" % result )
        
        result = self.getNeoBoardArticle()._edit( creator='wankyu'
                     , password='12345'
                     , creator_email='wankyu2@neoqst.com'
                     , title='Changed Title'
                     , text='Changed Text'
                     , category='Changed Category'
                     , REQUEST=REQUEST )              
                     
        self.failIf( result is not None ,  "Article modification not working! - %s" % result )
        
        self.failIf( self.getNeoBoardArticle().creator_email != 'wankyu2@neoqst.com' ,  "Article modification not working - creator_email not changed!")
        self.failIf( self.getNeoBoardArticle().title != 'Changed Title' ,  "Article modification not working - title not changed!")                     
        self.failIf( self.getNeoBoardArticle().text != 'Changed Text' ,  "Article modification not working - text not changed!")
        self.failIf( self.getNeoBoardArticle().category != 'Changed Category' ,  "Article modification not working - category not changed!")        
        
    def test_NeoBoardArticleAttachments( self ):
        article = self.getNeoBoardArticle()
        
        self.assertEqual( 3, article.getNumAttachments() )
        
        self.failUnless( article.getAttachment(id=1).getFileSize() > 0, "Attachment 1 upload failed!" )
        self.failUnless( article.getAttachment(id=2).getFileSize() > 0, "Attachment 2 upload failed!" )
        self.failUnless( article.getAttachment(id=3).getFileSize() > 0, "Attachment 3 upload failed!" )
        
        self.failUnless( os.path.isfile(article.getAttachment(id=1).getFilePath()),  "Attachment 1 not saved on fs!")
        self.failUnless( os.path.isfile(article.getAttachment(id=2).getFilePath()),  "Attachment 2 not saved on fs!")
        self.failUnless( os.path.isfile(article.getAttachment(id=3).getFilePath()),  "Attachment 3 not saved on fs!")        

    def test_NeoBoardArticleAttachmentsCopy( self ):


        #simulate article copy
        
        if not os.path.isdir(ARCHIVE_FILE_PATH):
            os.mkdir(ARCHIVE_FILE_PATH, 0755)

        if not os.path.isdir(ARCHIVE_FILE_PATH2):
            os.mkdir(ARCHIVE_FILE_PATH2, 0755)
        
       
        self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH2
       
        article = self.getNeoBoardArticle() 
        article.getAttachment(id=1)._checkFile()
        article.getAttachment(id=2)._checkFile()
        article.getAttachment(id=3)._checkFile()

        article.getAttachment(id=1)._resetFilePath()
        article.getAttachment(id=2)._resetFilePath()
        article.getAttachment(id=3)._resetFilePath()        

        file1 = article.getAttachment(id=1).getFilePath()
        file2 = article.getAttachment(id=2).getFilePath()
        file3 = article.getAttachment(id=3).getFilePath()
        self.failUnless( os.path.isfile(file1),  "Attachment 1 copy failed! - %s" % file1)
        self.failUnless( os.path.isfile(file2),  "Attachment 2 copy failed! - %s" % file2)
        self.failUnless( os.path.isfile(file3),  "Attachment 3 copy failed! - %s" % file3)        
        
        self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH
        
        article.getAttachment(id=1)._resetFilePath()
        article.getAttachment(id=2)._resetFilePath()
        article.getAttachment(id=3)._resetFilePath()
        
        file1 = article.getAttachment(id=1).getFilePath()
        file2 = article.getAttachment(id=2).getFilePath()
        file3 = article.getAttachment(id=3).getFilePath()
        self.failUnless( os.path.isfile(file1),  "Attachment 1 original disappeared! - %s" % file1)
        self.failUnless( os.path.isfile(file2),  "Attachment 2 original disappeared! - %s" % file2)
        self.failUnless( os.path.isfile(file3),  "Attachment 3 original disappeared! - %s" % file3)

    def test_NeoBoardArticleAttachmentsMove( self ):
        
        # simulate move
        article = self.getNeoBoardArticle()
        self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH
        
        article.getAttachment(id=1)._resetFilePath()
        article.getAttachment(id=2)._resetFilePath()
        article.getAttachment(id=3)._resetFilePath()        
        
        article.getAttachment(id=1)._make_temp_file()
        article.getAttachment(id=2)._make_temp_file()
        article.getAttachment(id=3)._make_temp_file()

        self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH2
        
        
        article.getAttachment(id=1)._checkFile()
        article.getAttachment(id=2)._checkFile()
        article.getAttachment(id=3)._checkFile()             
        
        article.getAttachment(id=1)._resetFilePath()
        article.getAttachment(id=2)._resetFilePath()
        article.getAttachment(id=3)._resetFilePath()        

        file1 = article.getAttachment(id=1).getFilePath()
        file2 = article.getAttachment(id=2).getFilePath()
        file3 = article.getAttachment(id=3).getFilePath()
        self.failUnless( os.path.isfile(file1),  "Attachment 1 move failed! - %s" % file1)
        self.failUnless( os.path.isfile(file2),  "Attachment 2 move failed! - %s" % file2)
        self.failUnless( os.path.isfile(file3),  "Attachment 3 move failed! - %s" % file3)        
        
        self.getNeoBoard().archive_path = ARCHIVE_FILE_PATH       
        
        article.getAttachment(id=1)._resetFilePath()
        article.getAttachment(id=2)._resetFilePath()
        article.getAttachment(id=3)._resetFilePath()        
        
        file1 = article.getAttachment(id=1).getFilePath()
        file2 = article.getAttachment(id=2).getFilePath()
        file3 = article.getAttachment(id=3).getFilePath()
        self.failIf( os.path.isfile(file1),  "Attachment 1 original remains! - %s" % file1)
        self.failIf( os.path.isfile(file2),  "Attachment 2 original remains! - %s" % file2)
        self.failIf( os.path.isfile(file3),  "Attachment 3 original remains! - %s" % file3)

    def test_NeoBoardCatalog( self ):
        
        self.getCatalog().indexObject(self.getNeoBoardArticle())
        results = self.getCatalog()(Creator='wankyu')
    
        self.failIf( len(results) < 1 ,  "Catalog not working!")
        
    def test_NeoBoardComment( self ):
        self.failUnless( self.getComment().Creator() == 'wankyu',  "Can't instantiate NeoBoard Article Comment!")  
        
        view = self.getNeoBoardArticle().getRenderedNeoPortalContent( view_comment=1 )
        
        self.failIf( view.find( 'RENDERED' ) == -1 ,  "Inline comment password check not working! - %s" % view )
        
        result = self.getNeoBoardArticle().manage_modifyNeoBoardArticleComment( comment_id=NEOBOARD_REQUEST_COMMENT_PREFIX+'1'
                                                                    , creator='wankyu'
                                                                    , password='123'
                                                                    , creator_email='wankyu@neoqst.com'
                                                                    , text='This is'
                                                                    , REQUEST=REQUEST )

        self.failIf( result is None ,  "Inline comment password check not working! - %s" % result )
        
        result = self.getComment().manage_modifyNeoBoardArticleComment( creator='wankyu'
                                                                    , password='12345'
                                                                    , creator_email='wankyu2@neoqst.com'
                                                                    , text='Inline comment changed!'
                                                                    , REQUEST=REQUEST )

        self.failIf( not result.endswith( self.getComment().getId() ),  "Inline comment modification not working! - %s" % result )
        self.failIf( self.getComment().creator_email != 'wankyu2@neoqst.com' ,  "Inline comment modification not working - creator_email not changed!")
        self.failIf( self.getComment().text != 'Inline comment changed!' ,  "Inline comment modification not working - text not changed!")

        
def test_suite():
    return TestSuite((
        makeSuite( NeoBoardTests ),
        ))

if __name__ == '__main__':
    main(defaultTest='test_suite')
